#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 4.0.1-9346c8cc45 (http://hl7.org/fhir/StructureDefinition/PaymentReconciliation) on 2022-06-22.
#  2022, SMART Health IT.


from . import domainresource

class PaymentReconciliation(domainresource.DomainResource):
    """ PaymentReconciliation resource.
    
    This resource provides the details including amount of a payment and
    allocates the payment items being paid.
    """
    
    resource_type = "PaymentReconciliation"

    _attribute_docstrings = {}
    """ Dictionary of attribute documentation."""
    _attribute_docstrings['identifier'] = """Business Identifier for a payment reconciliation."""
    _attribute_docstrings['status'] = """The status of the resource instance."""
    _attribute_docstrings['period'] = """Period covered."""
    _attribute_docstrings['created'] = """Creation date."""
    _attribute_docstrings['paymentIssuer'] = """Party generating payment."""
    _attribute_docstrings['request'] = """Reference to requesting resource."""
    _attribute_docstrings['requestor'] = """Responsible practitioner."""
    _attribute_docstrings['outcome'] = """The outcome of a request for a reconciliation."""
    _attribute_docstrings['disposition'] = """Disposition message."""
    _attribute_docstrings['paymentDate'] = """When payment issued."""
    _attribute_docstrings['paymentAmount'] = """Total amount of Payment."""
    _attribute_docstrings['paymentIdentifier'] = """Business identifier for the payment."""
    _attribute_docstrings['detail'] = """Settlement particulars."""
    _attribute_docstrings['formCode'] = """Printed form identifier."""
    _attribute_docstrings['processNote'] = """Note concerning processing."""

    @classmethod
    def attribute_docstrings(cls):
        """Get dict of attributes docstrings."""
        return cls._attribute_docstrings

    _attribute_enums = {}
    """ Dictionary of enum configuration."""
    _attribute_enums['status'] = {
        'url': 'http://hl7.org/fhir/fm-status',
        'restricted_to': ['active', 'cancelled', 'draft', 'entered-in-error'],
        'binding_strength': 'required',
        'class_name': 'str'
    }
    _attribute_enums['outcome'] = {
        'url': 'http://hl7.org/fhir/remittance-outcome',
        'restricted_to': ['queued', 'complete', 'error', 'partial'],
        'binding_strength': 'required',
        'class_name': 'str'
    }

    @classmethod
    def attribute_enums(cls):
        """Get dict of attributes with enums, Code or CodeableConcept."""
        return cls._attribute_enums

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.identifier = None
        """ Business Identifier for a payment reconciliation.
        List of `Identifier` items (represented as `dict` in JSON). """
        
        self.status = None
        """ The status of the resource instance.
        Type `str`. """
        
        self.period = None
        """ Period covered.
        Type `Period` (represented as `dict` in JSON). """
        
        self.created = None
        """ Creation date.
        Type `FHIRDate` (represented as `str` in JSON). """
        
        self.paymentIssuer = None
        """ Party generating payment.
        Type `FHIRReference` (represented as `dict` in JSON). """
        
        self.request = None
        """ Reference to requesting resource.
        Type `FHIRReference` (represented as `dict` in JSON). """
        
        self.requestor = None
        """ Responsible practitioner.
        Type `FHIRReference` (represented as `dict` in JSON). """
        
        self.outcome = None
        """ The outcome of a request for a reconciliation.
        Type `str`. """
        
        self.disposition = None
        """ Disposition message.
        Type `str`. """
        
        self.paymentDate = None
        """ When payment issued.
        Type `FHIRDate` (represented as `str` in JSON). """
        
        self.paymentAmount = None
        """ Total amount of Payment.
        Type `Money` (represented as `dict` in JSON). """
        
        self.paymentIdentifier = None
        """ Business identifier for the payment.
        Type `Identifier` (represented as `dict` in JSON). """
        
        self.detail = None
        """ Settlement particulars.
        List of `PaymentReconciliationDetail` items (represented as `dict` in JSON). """
        
        self.formCode = None
        """ Printed form identifier.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.processNote = None
        """ Note concerning processing.
        List of `PaymentReconciliationProcessNote` items (represented as `dict` in JSON). """
        
        super(PaymentReconciliation, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(PaymentReconciliation, self).elementProperties()
        js.extend([
            ("identifier", "identifier", identifier.Identifier, True, None, False),
            ("status", "status", str, False, None, True),
            ("period", "period", period.Period, False, None, False),
            ("created", "created", fhirdate.FHIRDate, False, None, True),
            ("paymentIssuer", "paymentIssuer", fhirreference.FHIRReference, False, None, False),
            ("request", "request", fhirreference.FHIRReference, False, None, False),
            ("requestor", "requestor", fhirreference.FHIRReference, False, None, False),
            ("outcome", "outcome", str, False, None, False),
            ("disposition", "disposition", str, False, None, False),
            ("paymentDate", "paymentDate", fhirdate.FHIRDate, False, None, True),
            ("paymentAmount", "paymentAmount", money.Money, False, None, True),
            ("paymentIdentifier", "paymentIdentifier", identifier.Identifier, False, None, False),
            ("detail", "detail", PaymentReconciliationDetail, True, None, False),
            ("formCode", "formCode", codeableconcept.CodeableConcept, False, None, False),
            ("processNote", "processNote", PaymentReconciliationProcessNote, True, None, False),
        ])
        return js


from . import backboneelement

class PaymentReconciliationDetail(backboneelement.BackboneElement):
    """ Settlement particulars.
    
    Distribution of the payment amount for a previously acknowledged payable.
    """

    _attribute_docstrings = {}
    """ Dictionary of attribute documentation."""
    _attribute_docstrings['identifier'] = """Business identifier of the payment detail."""
    _attribute_docstrings['predecessor'] = """Business identifier of the prior payment detail."""
    _attribute_docstrings['type'] = """Code to indicate the nature of the payment."""
    _attribute_docstrings['request'] = """Request giving rise to the payment."""
    _attribute_docstrings['submitter'] = """Submitter of the request."""
    _attribute_docstrings['response'] = """Response committing to a payment."""
    _attribute_docstrings['date'] = """Date of commitment to pay."""
    _attribute_docstrings['responsible'] = """Contact for the response."""
    _attribute_docstrings['payee'] = """Recipient of the payment."""
    _attribute_docstrings['amount'] = """Amount allocated to this payable."""

    @classmethod
    def attribute_docstrings(cls):
        """Get dict of attributes docstrings."""
        return cls._attribute_docstrings

    _attribute_enums = {}
    """ Dictionary of enum configuration."""
    _attribute_enums['type'] = {
        'url': 'http://terminology.hl7.org/CodeSystem/payment-type',
        'restricted_to': ['payment', 'adjustment', 'advance'],
        'binding_strength': 'example',
        'class_name': 'CodeableConcept'
    }

    @classmethod
    def attribute_enums(cls):
        """Get dict of attributes with enums, Code or CodeableConcept."""
        return cls._attribute_enums

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.identifier = None
        """ Business identifier of the payment detail.
        Type `Identifier` (represented as `dict` in JSON). """
        
        self.predecessor = None
        """ Business identifier of the prior payment detail.
        Type `Identifier` (represented as `dict` in JSON). """
        
        self.type = None
        """ Code to indicate the nature of the payment.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.request = None
        """ Request giving rise to the payment.
        Type `FHIRReference` (represented as `dict` in JSON). """
        
        self.submitter = None
        """ Submitter of the request.
        Type `FHIRReference` (represented as `dict` in JSON). """
        
        self.response = None
        """ Response committing to a payment.
        Type `FHIRReference` (represented as `dict` in JSON). """
        
        self.date = None
        """ Date of commitment to pay.
        Type `FHIRDate` (represented as `str` in JSON). """
        
        self.responsible = None
        """ Contact for the response.
        Type `FHIRReference` (represented as `dict` in JSON). """
        
        self.payee = None
        """ Recipient of the payment.
        Type `FHIRReference` (represented as `dict` in JSON). """
        
        self.amount = None
        """ Amount allocated to this payable.
        Type `Money` (represented as `dict` in JSON). """
        
        super(PaymentReconciliationDetail, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(PaymentReconciliationDetail, self).elementProperties()
        js.extend([
            ("identifier", "identifier", identifier.Identifier, False, None, False),
            ("predecessor", "predecessor", identifier.Identifier, False, None, False),
            ("type", "type", codeableconcept.CodeableConcept, False, None, True),
            ("request", "request", fhirreference.FHIRReference, False, None, False),
            ("submitter", "submitter", fhirreference.FHIRReference, False, None, False),
            ("response", "response", fhirreference.FHIRReference, False, None, False),
            ("date", "date", fhirdate.FHIRDate, False, None, False),
            ("responsible", "responsible", fhirreference.FHIRReference, False, None, False),
            ("payee", "payee", fhirreference.FHIRReference, False, None, False),
            ("amount", "amount", money.Money, False, None, False),
        ])
        return js


class PaymentReconciliationProcessNote(backboneelement.BackboneElement):
    """ Note concerning processing.
    
    A note that describes or explains the processing in a human readable form.
    """

    _attribute_docstrings = {}
    """ Dictionary of attribute documentation."""
    _attribute_docstrings['type'] = """The business purpose of the note text."""
    _attribute_docstrings['text'] = """Note explanatory text."""

    @classmethod
    def attribute_docstrings(cls):
        """Get dict of attributes docstrings."""
        return cls._attribute_docstrings

    _attribute_enums = {}
    """ Dictionary of enum configuration."""
    _attribute_enums['type'] = {
        'url': 'http://hl7.org/fhir/note-type',
        'restricted_to': ['display', 'print', 'printoper'],
        'binding_strength': 'required',
        'class_name': 'str'
    }

    @classmethod
    def attribute_enums(cls):
        """Get dict of attributes with enums, Code or CodeableConcept."""
        return cls._attribute_enums

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.type = None
        """ The business purpose of the note text.
        Type `str`. """
        
        self.text = None
        """ Note explanatory text.
        Type `str`. """
        
        super(PaymentReconciliationProcessNote, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(PaymentReconciliationProcessNote, self).elementProperties()
        js.extend([
            ("type", "type", str, False, None, False),
            ("text", "text", str, False, None, False),
        ])
        return js


import sys
try:
    from . import codeableconcept
except ImportError:
    codeableconcept = sys.modules[__package__ + '.codeableconcept']
try:
    from . import fhirdate
except ImportError:
    fhirdate = sys.modules[__package__ + '.fhirdate']
try:
    from . import fhirreference
except ImportError:
    fhirreference = sys.modules[__package__ + '.fhirreference']
try:
    from . import identifier
except ImportError:
    identifier = sys.modules[__package__ + '.identifier']
try:
    from . import money
except ImportError:
    money = sys.modules[__package__ + '.money']
try:
    from . import period
except ImportError:
    period = sys.modules[__package__ + '.period']



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 4.0.1-9346c8cc45 (http://hl7.org/fhir/StructureDefinition/DiagnosticReport) on 2022-06-22.
#  2022, SMART Health IT.


from . import domainresource

class DiagnosticReport(domainresource.DomainResource):
    """ A Diagnostic report - a combination of request information, atomic results,
    images, interpretation, as well as formatted reports.
    
    The findings and interpretation of diagnostic  tests performed on patients,
    groups of patients, devices, and locations, and/or specimens derived from
    these. The report includes clinical context such as requesting and provider
    information, and some mix of atomic results, images, textual and coded
    interpretations, and formatted representation of diagnostic reports.
    """
    
    resource_type = "DiagnosticReport"

    _attribute_docstrings = {}
    """ Dictionary of attribute documentation."""
    _attribute_docstrings['identifier'] = """Business identifier for report."""
    _attribute_docstrings['basedOn'] = """What was requested."""
    _attribute_docstrings['status'] = """The status of the diagnostic report."""
    _attribute_docstrings['category'] = """Service category."""
    _attribute_docstrings['code'] = """Name/Code for this diagnostic report."""
    _attribute_docstrings['subject'] = """The subject of the report - usually, but not always, the patient."""
    _attribute_docstrings['encounter'] = """Health care event when test ordered."""
    _attribute_docstrings['effectiveDateTime'] = """Clinically relevant time/time-period for report."""
    _attribute_docstrings['effectivePeriod'] = """Clinically relevant time/time-period for report."""
    _attribute_docstrings['issued'] = """DateTime this version was made."""
    _attribute_docstrings['performer'] = """Responsible Diagnostic Service."""
    _attribute_docstrings['resultsInterpreter'] = """Primary result interpreter."""
    _attribute_docstrings['specimen'] = """Specimens this report is based on."""
    _attribute_docstrings['result'] = """Observations."""
    _attribute_docstrings['imagingStudy'] = """Reference to full details of imaging associated with the diagnostic report."""
    _attribute_docstrings['media'] = """Key images associated with this report."""
    _attribute_docstrings['conclusion'] = """Clinical conclusion (interpretation) of test results."""
    _attribute_docstrings['conclusionCode'] = """Codes for the clinical conclusion of test results."""
    _attribute_docstrings['presentedForm'] = """Entire report as issued."""

    @classmethod
    def attribute_docstrings(cls):
        """Get dict of attributes docstrings."""
        return cls._attribute_docstrings

    _attribute_enums = {}
    """ Dictionary of enum configuration."""
    _attribute_enums['status'] = {
        'url': 'http://hl7.org/fhir/diagnostic-report-status',
        'restricted_to': ['registered', 'partial', 'preliminary', 'final', 'amended', 'corrected', 'appended', 'cancelled', 'entered-in-error', 'unknown'],
        'binding_strength': 'required',
        'class_name': 'str'
    }

    @classmethod
    def attribute_enums(cls):
        """Get dict of attributes with enums, Code or CodeableConcept."""
        return cls._attribute_enums

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.identifier = None
        """ Business identifier for report.
        List of `Identifier` items (represented as `dict` in JSON). """
        
        self.basedOn = None
        """ What was requested.
        List of `FHIRReference` items (represented as `dict` in JSON). """
        
        self.status = None
        """ The status of the diagnostic report.
        Type `str`. """
        
        self.category = None
        """ Service category.
        List of `CodeableConcept` items (represented as `dict` in JSON). """
        
        self.code = None
        """ Name/Code for this diagnostic report.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.subject = None
        """ The subject of the report - usually, but not always, the patient.
        Type `FHIRReference` (represented as `dict` in JSON). """
        
        self.encounter = None
        """ Health care event when test ordered.
        Type `FHIRReference` (represented as `dict` in JSON). """
        
        self.effectiveDateTime = None
        """ Clinically relevant time/time-period for report.
        Type `FHIRDate` (represented as `str` in JSON). """
        
        self.effectivePeriod = None
        """ Clinically relevant time/time-period for report.
        Type `Period` (represented as `dict` in JSON). """
        
        self.issued = None
        """ DateTime this version was made.
        Type `FHIRDate` (represented as `str` in JSON). """
        
        self.performer = None
        """ Responsible Diagnostic Service.
        List of `FHIRReference` items (represented as `dict` in JSON). """
        
        self.resultsInterpreter = None
        """ Primary result interpreter.
        List of `FHIRReference` items (represented as `dict` in JSON). """
        
        self.specimen = None
        """ Specimens this report is based on.
        List of `FHIRReference` items (represented as `dict` in JSON). """
        
        self.result = None
        """ Observations.
        List of `FHIRReference` items (represented as `dict` in JSON). """
        
        self.imagingStudy = None
        """ Reference to full details of imaging associated with the diagnostic
        report.
        List of `FHIRReference` items (represented as `dict` in JSON). """
        
        self.media = None
        """ Key images associated with this report.
        List of `DiagnosticReportMedia` items (represented as `dict` in JSON). """
        
        self.conclusion = None
        """ Clinical conclusion (interpretation) of test results.
        Type `str`. """
        
        self.conclusionCode = None
        """ Codes for the clinical conclusion of test results.
        List of `CodeableConcept` items (represented as `dict` in JSON). """
        
        self.presentedForm = None
        """ Entire report as issued.
        List of `Attachment` items (represented as `dict` in JSON). """
        
        super(DiagnosticReport, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(DiagnosticReport, self).elementProperties()
        js.extend([
            ("identifier", "identifier", identifier.Identifier, True, None, False),
            ("basedOn", "basedOn", fhirreference.FHIRReference, True, None, False),
            ("status", "status", str, False, None, True),
            ("category", "category", codeableconcept.CodeableConcept, True, None, False),
            ("code", "code", codeableconcept.CodeableConcept, False, None, True),
            ("subject", "subject", fhirreference.FHIRReference, False, None, False),
            ("encounter", "encounter", fhirreference.FHIRReference, False, None, False),
            ("effectiveDateTime", "effectiveDateTime", fhirdate.FHIRDate, False, "effective", False),
            ("effectivePeriod", "effectivePeriod", period.Period, False, "effective", False),
            ("issued", "issued", fhirdate.FHIRDate, False, None, False),
            ("performer", "performer", fhirreference.FHIRReference, True, None, False),
            ("resultsInterpreter", "resultsInterpreter", fhirreference.FHIRReference, True, None, False),
            ("specimen", "specimen", fhirreference.FHIRReference, True, None, False),
            ("result", "result", fhirreference.FHIRReference, True, None, False),
            ("imagingStudy", "imagingStudy", fhirreference.FHIRReference, True, None, False),
            ("media", "media", DiagnosticReportMedia, True, None, False),
            ("conclusion", "conclusion", str, False, None, False),
            ("conclusionCode", "conclusionCode", codeableconcept.CodeableConcept, True, None, False),
            ("presentedForm", "presentedForm", attachment.Attachment, True, None, False),
        ])
        return js


from . import backboneelement

class DiagnosticReportMedia(backboneelement.BackboneElement):
    """ Key images associated with this report.
    
    A list of key images associated with this report. The images are generally
    created during the diagnostic process, and may be directly of the patient,
    or of treated specimens (i.e. slides of interest).
    """

    _attribute_docstrings = {}
    """ Dictionary of attribute documentation."""
    _attribute_docstrings['comment'] = """Comment about the image (e.g. explanation)."""
    _attribute_docstrings['link'] = """Reference to the image source."""

    @classmethod
    def attribute_docstrings(cls):
        """Get dict of attributes docstrings."""
        return cls._attribute_docstrings

    _attribute_enums = {}
    """ Dictionary of enum configuration."""

    @classmethod
    def attribute_enums(cls):
        """Get dict of attributes with enums, Code or CodeableConcept."""
        return cls._attribute_enums

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.comment = None
        """ Comment about the image (e.g. explanation).
        Type `str`. """
        
        self.link = None
        """ Reference to the image source.
        Type `FHIRReference` (represented as `dict` in JSON). """
        
        super(DiagnosticReportMedia, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(DiagnosticReportMedia, self).elementProperties()
        js.extend([
            ("comment", "comment", str, False, None, False),
            ("link", "link", fhirreference.FHIRReference, False, None, True),
        ])
        return js


import sys
try:
    from . import attachment
except ImportError:
    attachment = sys.modules[__package__ + '.attachment']
try:
    from . import codeableconcept
except ImportError:
    codeableconcept = sys.modules[__package__ + '.codeableconcept']
try:
    from . import fhirdate
except ImportError:
    fhirdate = sys.modules[__package__ + '.fhirdate']
try:
    from . import fhirreference
except ImportError:
    fhirreference = sys.modules[__package__ + '.fhirreference']
try:
    from . import identifier
except ImportError:
    identifier = sys.modules[__package__ + '.identifier']
try:
    from . import period
except ImportError:
    period = sys.modules[__package__ + '.period']



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 4.0.1-9346c8cc45 (http://hl7.org/fhir/StructureDefinition/Dosage) on 2022-06-22.
#  2022, SMART Health IT.


from . import backboneelement

class Dosage(backboneelement.BackboneElement):
    """ How the medication is/was taken or should be taken.
    
    Indicates how the medication is/was taken or should be taken by the
    patient.
    """

    _attribute_docstrings = {}
    """ Dictionary of attribute documentation."""
    _attribute_docstrings['sequence'] = """The order of the dosage instructions."""
    _attribute_docstrings['text'] = """Free text dosage instructions e.g. SIG."""
    _attribute_docstrings['additionalInstruction'] = """Supplemental instruction or warnings to the patient - e.g. "with meals", "may cause drowsiness"."""
    _attribute_docstrings['patientInstruction'] = """Patient or consumer oriented instructions."""
    _attribute_docstrings['timing'] = """When medication should be administered."""
    _attribute_docstrings['asNeededBoolean'] = """Take "as needed" (for x)."""
    _attribute_docstrings['asNeededCodeableConcept'] = """Take "as needed" (for x)."""
    _attribute_docstrings['site'] = """Body site to administer to."""
    _attribute_docstrings['route'] = """How drug should enter body."""
    _attribute_docstrings['method'] = """Technique for administering medication."""
    _attribute_docstrings['doseAndRate'] = """Amount of medication administered."""
    _attribute_docstrings['maxDosePerPeriod'] = """Upper limit on medication per unit of time."""
    _attribute_docstrings['maxDosePerAdministration'] = """Upper limit on medication per administration."""
    _attribute_docstrings['maxDosePerLifetime'] = """Upper limit on medication per lifetime of the patient."""

    @classmethod
    def attribute_docstrings(cls):
        """Get dict of attributes docstrings."""
        return cls._attribute_docstrings

    _attribute_enums = {}
    """ Dictionary of enum configuration."""

    @classmethod
    def attribute_enums(cls):
        """Get dict of attributes with enums, Code or CodeableConcept."""
        return cls._attribute_enums

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.sequence = None
        """ The order of the dosage instructions.
        Type `int`. """
        
        self.text = None
        """ Free text dosage instructions e.g. SIG.
        Type `str`. """
        
        self.additionalInstruction = None
        """ Supplemental instruction or warnings to the patient - e.g. "with
        meals", "may cause drowsiness".
        List of `CodeableConcept` items (represented as `dict` in JSON). """
        
        self.patientInstruction = None
        """ Patient or consumer oriented instructions.
        Type `str`. """
        
        self.timing = None
        """ When medication should be administered.
        Type `Timing` (represented as `dict` in JSON). """
        
        self.asNeededBoolean = None
        """ Take "as needed" (for x).
        Type `bool`. """
        
        self.asNeededCodeableConcept = None
        """ Take "as needed" (for x).
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.site = None
        """ Body site to administer to.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.route = None
        """ How drug should enter body.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.method = None
        """ Technique for administering medication.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.doseAndRate = None
        """ Amount of medication administered.
        List of `DosageDoseAndRate` items (represented as `dict` in JSON). """
        
        self.maxDosePerPeriod = None
        """ Upper limit on medication per unit of time.
        Type `Ratio` (represented as `dict` in JSON). """
        
        self.maxDosePerAdministration = None
        """ Upper limit on medication per administration.
        Type `Quantity` (represented as `dict` in JSON). """
        
        self.maxDosePerLifetime = None
        """ Upper limit on medication per lifetime of the patient.
        Type `Quantity` (represented as `dict` in JSON). """
        
        super(Dosage, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(Dosage, self).elementProperties()
        js.extend([
            ("sequence", "sequence", int, False, None, False),
            ("text", "text", str, False, None, False),
            ("additionalInstruction", "additionalInstruction", codeableconcept.CodeableConcept, True, None, False),
            ("patientInstruction", "patientInstruction", str, False, None, False),
            ("timing", "timing", timing.Timing, False, None, False),
            ("asNeededBoolean", "asNeededBoolean", bool, False, "asNeeded", False),
            ("asNeededCodeableConcept", "asNeededCodeableConcept", codeableconcept.CodeableConcept, False, "asNeeded", False),
            ("site", "site", codeableconcept.CodeableConcept, False, None, False),
            ("route", "route", codeableconcept.CodeableConcept, False, None, False),
            ("method", "method", codeableconcept.CodeableConcept, False, None, False),
            ("doseAndRate", "doseAndRate", DosageDoseAndRate, True, None, False),
            ("maxDosePerPeriod", "maxDosePerPeriod", ratio.Ratio, False, None, False),
            ("maxDosePerAdministration", "maxDosePerAdministration", quantity.Quantity, False, None, False),
            ("maxDosePerLifetime", "maxDosePerLifetime", quantity.Quantity, False, None, False),
        ])
        return js


from . import element

class DosageDoseAndRate(element.Element):
    """ Amount of medication administered.
    
    The amount of medication administered.
    """

    _attribute_docstrings = {}
    """ Dictionary of attribute documentation."""
    _attribute_docstrings['type'] = """The kind of dose or rate specified, for example, ordered or calculated."""
    _attribute_docstrings['doseRange'] = """Amount of medication per dose."""
    _attribute_docstrings['doseQuantity'] = """Amount of medication per dose."""
    _attribute_docstrings['rateRatio'] = """Amount of medication per unit of time."""
    _attribute_docstrings['rateRange'] = """Amount of medication per unit of time."""
    _attribute_docstrings['rateQuantity'] = """Amount of medication per unit of time."""

    @classmethod
    def attribute_docstrings(cls):
        """Get dict of attributes docstrings."""
        return cls._attribute_docstrings

    _attribute_enums = {}
    """ Dictionary of enum configuration."""
    _attribute_enums['type'] = {
        'url': 'http://terminology.hl7.org/CodeSystem/dose-rate-type',
        'restricted_to': ['calculated', 'ordered'],
        'binding_strength': 'example',
        'class_name': 'CodeableConcept'
    }

    @classmethod
    def attribute_enums(cls):
        """Get dict of attributes with enums, Code or CodeableConcept."""
        return cls._attribute_enums

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.type = None
        """ The kind of dose or rate specified, for example, ordered or
        calculated.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.doseRange = None
        """ Amount of medication per dose.
        Type `Range` (represented as `dict` in JSON). """
        
        self.doseQuantity = None
        """ Amount of medication per dose.
        Type `Quantity` (represented as `dict` in JSON). """
        
        self.rateRatio = None
        """ Amount of medication per unit of time.
        Type `Ratio` (represented as `dict` in JSON). """
        
        self.rateRange = None
        """ Amount of medication per unit of time.
        Type `Range` (represented as `dict` in JSON). """
        
        self.rateQuantity = None
        """ Amount of medication per unit of time.
        Type `Quantity` (represented as `dict` in JSON). """
        
        super(DosageDoseAndRate, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(DosageDoseAndRate, self).elementProperties()
        js.extend([
            ("type", "type", codeableconcept.CodeableConcept, False, None, False),
            ("doseRange", "doseRange", range.Range, False, "dose", False),
            ("doseQuantity", "doseQuantity", quantity.Quantity, False, "dose", False),
            ("rateRatio", "rateRatio", ratio.Ratio, False, "rate", False),
            ("rateRange", "rateRange", range.Range, False, "rate", False),
            ("rateQuantity", "rateQuantity", quantity.Quantity, False, "rate", False),
        ])
        return js


import sys
try:
    from . import codeableconcept
except ImportError:
    codeableconcept = sys.modules[__package__ + '.codeableconcept']
try:
    from . import quantity
except ImportError:
    quantity = sys.modules[__package__ + '.quantity']
try:
    from . import range
except ImportError:
    range = sys.modules[__package__ + '.range']
try:
    from . import ratio
except ImportError:
    ratio = sys.modules[__package__ + '.ratio']
try:
    from . import timing
except ImportError:
    timing = sys.modules[__package__ + '.timing']



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 4.0.1-9346c8cc45 on 2022-06-22.
#  2022, SMART Health IT.

import io
import json
import logging
import os
import typing
import unittest

from . import testscript

from .fhirdate import FHIRDate
import logging


class TestScriptTests(unittest.TestCase):
    def instantiate_from(self, filename):
        datadir = os.environ.get('FHIR_UNITTEST_DATADIR') or ''
        with io.open(os.path.join(datadir, filename), 'r', encoding='utf-8') as handle:
            js = json.load(handle)
            self.assertEqual("TestScript", js["resourceType"])
        return testscript.TestScript(js)
    
    def testTestScript1(self):
        inst = self.instantiate_from("testscript-example-multisystem.json")
        self.assertIsNotNone(inst, "Must have instantiated a TestScript instance")
        self.implTestScript1(inst)
        
        js = inst.as_json()
        self.assertEqual("TestScript", js["resourceType"])
        inst2 = testscript.TestScript(js)
        self.implTestScript1(inst2)
        self.evaluate_simplified_json(inst2)
    
    def implTestScript1(self, inst):
        self.assertEqual(inst.contact[0].name, "Support")
        self.assertEqual(inst.contact[0].telecom[0].system, "email")
        self.assertEqual(inst.contact[0].telecom[0].use, "work")
        self.assertEqual(inst.contact[0].telecom[0].value, "support@HL7.org")
        self.assertEqual(inst.copyright, "© HL7.org 2011+")
        self.assertEqual(inst.date.date, FHIRDate("2017-01-18").date)
        self.assertEqual(inst.date.as_json(), "2017-01-18")
        self.assertEqual(inst.destination[0].index, 1)
        self.assertEqual(inst.destination[0].profile.code, "FHIR-Server")
        self.assertEqual(inst.destination[1].index, 2)
        self.assertEqual(inst.destination[1].profile.code, "FHIR-Server")
        self.assertTrue(inst.experimental)
        self.assertEqual(inst.id, "testscript-example-multisystem")
        self.assertEqual(inst.identifier.system, "urn:ietf:rfc:3986")
        self.assertEqual(inst.identifier.value, "urn:oid:1.3.6.1.4.1.21367.2005.3.7.9878")
        self.assertEqual(inst.jurisdiction[0].coding[0].code, "US")
        self.assertEqual(inst.jurisdiction[0].coding[0].display, "United States of America (the)")
        self.assertEqual(inst.jurisdiction[0].coding[0].system, "urn:iso:std:iso:3166")
        self.assertEqual(inst.metadata.capability[0].capabilities, "CapabilityStatement/example")
        self.assertEqual(inst.metadata.capability[0].description, "Patient Read Operation")
        self.assertEqual(inst.metadata.capability[0].destination, 1)
        self.assertEqual(inst.metadata.capability[0].link[0], "http://hl7.org/fhir/http.html#read")
        self.assertEqual(inst.metadata.capability[0].origin[0], 1)
        self.assertTrue(inst.metadata.capability[0].required)
        self.assertFalse(inst.metadata.capability[0].validated)
        self.assertEqual(inst.metadata.capability[1].capabilities, "CapabilityStatement/example")
        self.assertEqual(inst.metadata.capability[1].description, "Patient Read Operation")
        self.assertEqual(inst.metadata.capability[1].destination, 2)
        self.assertEqual(inst.metadata.capability[1].link[0], "http://hl7.org/fhir/http.html#read")
        self.assertEqual(inst.metadata.capability[1].origin[0], 1)
        self.assertTrue(inst.metadata.capability[1].required)
        self.assertFalse(inst.metadata.capability[1].validated)
        self.assertEqual(inst.metadata.link[0].description, "Demographics and other administrative information about an individual or animal receiving care or other health-related services.")
        self.assertEqual(inst.metadata.link[0].url, "http://hl7.org/fhir/patient.html")
        self.assertEqual(inst.name, "testscript-example-multisystem")
        self.assertEqual(inst.origin[0].index, 1)
        self.assertEqual(inst.origin[0].profile.code, "FHIR-Client")
        self.assertEqual(inst.publisher, "HL7")
        self.assertEqual(inst.purpose, "Patient Read Operation")
        self.assertEqual(inst.status, "draft")
        self.assertEqual(inst.test[0].action[0].operation.accept, "xml")
        self.assertEqual(inst.test[0].action[0].operation.contentType, "xml")
        self.assertEqual(inst.test[0].action[0].operation.description, "Read a Patient from the first destination test system and perform basic validation.")
        self.assertEqual(inst.test[0].action[0].operation.destination, 1)
        self.assertTrue(inst.test[0].action[0].operation.encodeRequestUrl)
        self.assertEqual(inst.test[0].action[0].operation.origin, 1)
        self.assertEqual(inst.test[0].action[0].operation.params, "/${Dest1PatientResourceId}")
        self.assertEqual(inst.test[0].action[0].operation.requestId, "request-read-patient-01")
        self.assertEqual(inst.test[0].action[0].operation.resource, "Patient")
        self.assertEqual(inst.test[0].action[0].operation.type.code, "read")
        self.assertEqual(inst.test[0].action[0].operation.type.system, "http://terminology.hl7.org/CodeSystem/testscript-operation-codes")
        self.assertEqual(inst.test[0].action[1].assert_fhir.description, "Confirm that the request method GET was sent by the client system under test.")
        self.assertEqual(inst.test[0].action[1].assert_fhir.requestMethod, "get")
        self.assertFalse(inst.test[0].action[1].assert_fhir.warningOnly)
        self.assertEqual(inst.test[0].action[2].assert_fhir.description, "Confirm that the client requested an Accept of xml.")
        self.assertEqual(inst.test[0].action[2].assert_fhir.direction, "request")
        self.assertEqual(inst.test[0].action[2].assert_fhir.headerField, "Accept")
        self.assertEqual(inst.test[0].action[2].assert_fhir.operator, "contains")
        self.assertEqual(inst.test[0].action[2].assert_fhir.value, "xml")
        self.assertFalse(inst.test[0].action[2].assert_fhir.warningOnly)
        self.assertEqual(inst.test[0].action[3].assert_fhir.description, "Confirm that the returned HTTP status is 200(OK).")
        self.assertEqual(inst.test[0].action[3].assert_fhir.direction, "response")
        self.assertEqual(inst.test[0].action[3].assert_fhir.response, "okay")
        self.assertFalse(inst.test[0].action[3].assert_fhir.warningOnly)
        self.assertEqual(inst.test[0].action[4].assert_fhir.contentType, "xml")
        self.assertEqual(inst.test[0].action[4].assert_fhir.description, "Confirm that the returned format is XML.")
        self.assertEqual(inst.test[0].action[4].assert_fhir.direction, "response")
        self.assertFalse(inst.test[0].action[4].assert_fhir.warningOnly)
        self.assertEqual(inst.test[0].action[5].assert_fhir.description, "Confirm that the returned resource type is Patient.")
        self.assertEqual(inst.test[0].action[5].assert_fhir.direction, "response")
        self.assertEqual(inst.test[0].action[5].assert_fhir.resource, "Patient")
        self.assertFalse(inst.test[0].action[5].assert_fhir.warningOnly)
        self.assertEqual(inst.test[0].description, "Read a Patient from the first destination test system using the user defined dynamic variable ${Dest1PatientResourceId}. Perform basic validation.")
        self.assertEqual(inst.test[0].id, "01-ReadPatient-Destination1")
        self.assertEqual(inst.test[0].name, "ReadPatient-Destination1")
        self.assertEqual(inst.test[1].action[0].operation.accept, "xml")
        self.assertEqual(inst.test[1].action[0].operation.contentType, "xml")
        self.assertEqual(inst.test[1].action[0].operation.description, "Read a Patient from the second destination test system and perform basic validation.")
        self.assertEqual(inst.test[1].action[0].operation.destination, 2)
        self.assertTrue(inst.test[1].action[0].operation.encodeRequestUrl)
        self.assertEqual(inst.test[1].action[0].operation.origin, 1)
        self.assertEqual(inst.test[1].action[0].operation.params, "/${Dest2PatientResourceId}")
        self.assertEqual(inst.test[1].action[0].operation.requestHeader[0].field, "Accept-Charset")
        self.assertEqual(inst.test[1].action[0].operation.requestHeader[0].value, "utf-8")
        self.assertEqual(inst.test[1].action[0].operation.resource, "Patient")
        self.assertEqual(inst.test[1].action[0].operation.type.code, "read")
        self.assertEqual(inst.test[1].action[0].operation.type.system, "http://terminology.hl7.org/CodeSystem/testscript-operation-codes")
        self.assertEqual(inst.test[1].action[1].assert_fhir.description, "Confirm that the client requested an Accept of xml.")
        self.assertEqual(inst.test[1].action[1].assert_fhir.direction, "request")
        self.assertEqual(inst.test[1].action[1].assert_fhir.headerField, "Accept")
        self.assertEqual(inst.test[1].action[1].assert_fhir.operator, "contains")
        self.assertEqual(inst.test[1].action[1].assert_fhir.value, "xml")
        self.assertFalse(inst.test[1].action[1].assert_fhir.warningOnly)
        self.assertEqual(inst.test[1].action[2].assert_fhir.description, "Confirm that the returned HTTP status is 200(OK).")
        self.assertEqual(inst.test[1].action[2].assert_fhir.direction, "response")
        self.assertEqual(inst.test[1].action[2].assert_fhir.response, "okay")
        self.assertFalse(inst.test[1].action[2].assert_fhir.warningOnly)
        self.assertEqual(inst.test[1].action[3].assert_fhir.contentType, "xml")
        self.assertEqual(inst.test[1].action[3].assert_fhir.description, "Confirm that the returned format is XML.")
        self.assertEqual(inst.test[1].action[3].assert_fhir.direction, "response")
        self.assertFalse(inst.test[1].action[3].assert_fhir.warningOnly)
        self.assertEqual(inst.test[1].action[4].assert_fhir.description, "Confirm that the returned resource type is Patient.")
        self.assertEqual(inst.test[1].action[4].assert_fhir.direction, "response")
        self.assertEqual(inst.test[1].action[4].assert_fhir.resource, "Patient")
        self.assertFalse(inst.test[1].action[4].assert_fhir.warningOnly)
        self.assertEqual(inst.test[1].description, "Read a Patient from the second destination test system using the user defined dynamic variable ${Dest2PatientResourceId}. Perform basic validation.")
        self.assertEqual(inst.test[1].id, "02-ReadPatient-Destination2")
        self.assertEqual(inst.test[1].name, "ReadPatient-Destination2")
        self.assertEqual(inst.text.status, "generated")
        self.assertEqual(inst.title, "Multisystem Test Script")
        self.assertEqual(inst.url, "http://hl7.org/fhir/TestScript/testscript-example-multisystem")
        self.assertEqual(inst.variable[0].defaultValue, "example")
        self.assertEqual(inst.variable[0].name, "Dest1PatientResourceId")
        self.assertEqual(inst.variable[1].defaultValue, "example")
        self.assertEqual(inst.variable[1].name, "Dest2PatientResourceId")
        self.assertEqual(inst.version, "1.0")
    
    def testTestScript2(self):
        inst = self.instantiate_from("testscript-example-history.json")
        self.assertIsNotNone(inst, "Must have instantiated a TestScript instance")
        self.implTestScript2(inst)
        
        js = inst.as_json()
        self.assertEqual("TestScript", js["resourceType"])
        inst2 = testscript.TestScript(js)
        self.implTestScript2(inst2)
        self.evaluate_simplified_json(inst2)
    
    def implTestScript2(self, inst):
        self.assertEqual(inst.contact[0].name, "Support")
        self.assertEqual(inst.contact[0].telecom[0].system, "email")
        self.assertEqual(inst.contact[0].telecom[0].use, "work")
        self.assertEqual(inst.contact[0].telecom[0].value, "support@HL7.org")
        self.assertEqual(inst.copyright, "© HL7.org 2011+")
        self.assertEqual(inst.date.date, FHIRDate("2017-01-18").date)
        self.assertEqual(inst.date.as_json(), "2017-01-18")
        self.assertTrue(inst.experimental)
        self.assertFalse(inst.fixture[0].autocreate)
        self.assertFalse(inst.fixture[0].autodelete)
        self.assertEqual(inst.fixture[0].id, "fixture-patient-create")
        self.assertFalse(inst.fixture[1].autocreate)
        self.assertFalse(inst.fixture[1].autodelete)
        self.assertEqual(inst.fixture[1].id, "fixture-patient-update")
        self.assertEqual(inst.id, "testscript-example-history")
        self.assertEqual(inst.identifier.system, "urn:ietf:rfc:3986")
        self.assertEqual(inst.identifier.value, "urn:oid:1.3.6.1.4.1.21367.2005.3.7.9877")
        self.assertEqual(inst.jurisdiction[0].coding[0].code, "US")
        self.assertEqual(inst.jurisdiction[0].coding[0].display, "United States of America (the)")
        self.assertEqual(inst.jurisdiction[0].coding[0].system, "urn:iso:std:iso:3166")
        self.assertEqual(inst.metadata.capability[0].capabilities, "CapabilityStatement/example")
        self.assertEqual(inst.metadata.capability[0].description, "Patient Update, Delete and History (Instance) Operations")
        self.assertEqual(inst.metadata.capability[0].link[0], "http://hl7.org/fhir/http.html#update")
        self.assertEqual(inst.metadata.capability[0].link[1], "http://hl7.org/fhir/http.html#delete")
        self.assertEqual(inst.metadata.capability[0].link[2], "http://hl7.org/fhir/http.html#history")
        self.assertTrue(inst.metadata.capability[0].required)
        self.assertFalse(inst.metadata.capability[0].validated)
        self.assertEqual(inst.metadata.link[0].description, "Demographics and other administrative information about an individual or animal receiving care or other health-related services.")
        self.assertEqual(inst.metadata.link[0].url, "http://hl7.org/fhir/patient.html")
        self.assertEqual(inst.name, "TestScript Example History")
        self.assertEqual(inst.publisher, "HL7")
        self.assertEqual(inst.purpose, "Patient (Conditional) Create, Update, Delete and History (Instance) Operations")
        self.assertEqual(inst.setup.action[0].operation.accept, "json")
        self.assertEqual(inst.setup.action[0].operation.description, "Execute a delete operation to insure the patient does not exist on the server.")
        self.assertTrue(inst.setup.action[0].operation.encodeRequestUrl)
        self.assertEqual(inst.setup.action[0].operation.label, "SetupDeletePatient")
        self.assertEqual(inst.setup.action[0].operation.params, "/${createResourceId}")
        self.assertEqual(inst.setup.action[0].operation.resource, "Patient")
        self.assertEqual(inst.setup.action[0].operation.type.code, "delete")
        self.assertEqual(inst.setup.action[0].operation.type.system, "http://terminology.hl7.org/CodeSystem/testscript-operation-codes")
        self.assertEqual(inst.setup.action[1].assert_fhir.description, "Confirm that the returned HTTP status is 200(OK) or 204(No Content).")
        self.assertEqual(inst.setup.action[1].assert_fhir.direction, "response")
        self.assertEqual(inst.setup.action[1].assert_fhir.operator, "in")
        self.assertEqual(inst.setup.action[1].assert_fhir.responseCode, "200,204")
        self.assertFalse(inst.setup.action[1].assert_fhir.warningOnly)
        self.assertEqual(inst.setup.action[2].operation.accept, "json")
        self.assertEqual(inst.setup.action[2].operation.contentType, "json")
        self.assertEqual(inst.setup.action[2].operation.description, "Create patient resource on test server using the contents of fixture-patient-create")
        self.assertTrue(inst.setup.action[2].operation.encodeRequestUrl)
        self.assertEqual(inst.setup.action[2].operation.label, "SetupCreatePatient")
        self.assertEqual(inst.setup.action[2].operation.params, "/${createResourceId}")
        self.assertEqual(inst.setup.action[2].operation.resource, "Patient")
        self.assertEqual(inst.setup.action[2].operation.sourceId, "fixture-patient-create")
        self.assertEqual(inst.setup.action[2].operation.type.code, "update")
        self.assertEqual(inst.setup.action[2].operation.type.system, "http://terminology.hl7.org/CodeSystem/testscript-operation-codes")
        self.assertEqual(inst.setup.action[3].assert_fhir.description, "Confirm that the returned HTTP status is 201(Created).")
        self.assertEqual(inst.setup.action[3].assert_fhir.direction, "response")
        self.assertEqual(inst.setup.action[3].assert_fhir.responseCode, "201")
        self.assertFalse(inst.setup.action[3].assert_fhir.warningOnly)
        self.assertEqual(inst.setup.action[4].operation.accept, "json")
        self.assertEqual(inst.setup.action[4].operation.contentType, "json")
        self.assertEqual(inst.setup.action[4].operation.description, "Update patient resource on test server using the contents of fixture-patient-update")
        self.assertTrue(inst.setup.action[4].operation.encodeRequestUrl)
        self.assertEqual(inst.setup.action[4].operation.label, "SetupUpdatePatient")
        self.assertEqual(inst.setup.action[4].operation.params, "/${createResourceId}")
        self.assertEqual(inst.setup.action[4].operation.resource, "Patient")
        self.assertEqual(inst.setup.action[4].operation.sourceId, "fixture-patient-update")
        self.assertEqual(inst.setup.action[4].operation.type.code, "update")
        self.assertEqual(inst.setup.action[4].operation.type.system, "http://terminology.hl7.org/CodeSystem/testscript-operation-codes")
        self.assertEqual(inst.setup.action[5].assert_fhir.description, "Confirm that the returned HTTP status is 200(OK).")
        self.assertEqual(inst.setup.action[5].assert_fhir.direction, "response")
        self.assertEqual(inst.setup.action[5].assert_fhir.responseCode, "200")
        self.assertFalse(inst.setup.action[5].assert_fhir.warningOnly)
        self.assertEqual(inst.status, "draft")
        self.assertEqual(inst.test[0].action[0].operation.accept, "json")
        self.assertEqual(inst.test[0].action[0].operation.contentType, "json")
        self.assertEqual(inst.test[0].action[0].operation.description, "Get the Patient history on the test server using the id from fixture-patient-create.")
        self.assertTrue(inst.test[0].action[0].operation.encodeRequestUrl)
        self.assertEqual(inst.test[0].action[0].operation.resource, "Patient")
        self.assertEqual(inst.test[0].action[0].operation.targetId, "fixture-patient-create")
        self.assertEqual(inst.test[0].action[0].operation.type.code, "history")
        self.assertEqual(inst.test[0].action[0].operation.type.system, "http://terminology.hl7.org/CodeSystem/testscript-operation-codes")
        self.assertEqual(inst.test[0].action[1].assert_fhir.description, "Confirm that the returned HTTP status is 200(OK).")
        self.assertEqual(inst.test[0].action[1].assert_fhir.direction, "response")
        self.assertEqual(inst.test[0].action[1].assert_fhir.response, "okay")
        self.assertFalse(inst.test[0].action[1].assert_fhir.warningOnly)
        self.assertEqual(inst.test[0].action[2].assert_fhir.description, "Confirm that the returned resource type is Bundle.")
        self.assertEqual(inst.test[0].action[2].assert_fhir.resource, "Bundle")
        self.assertFalse(inst.test[0].action[2].assert_fhir.warningOnly)
        self.assertEqual(inst.test[0].action[3].assert_fhir.description, "Confirm that the returned Bundle conforms to the base FHIR specification.")
        self.assertEqual(inst.test[0].action[3].assert_fhir.validateProfileId, "bundle-profile")
        self.assertFalse(inst.test[0].action[3].assert_fhir.warningOnly)
        self.assertEqual(inst.test[0].action[4].assert_fhir.description, "Confirm that the returned Bundle type equals 'history'.")
        self.assertEqual(inst.test[0].action[4].assert_fhir.operator, "equals")
        self.assertEqual(inst.test[0].action[4].assert_fhir.path, "fhir:Bundle/fhir:type/@value")
        self.assertEqual(inst.test[0].action[4].assert_fhir.value, "history")
        self.assertFalse(inst.test[0].action[4].assert_fhir.warningOnly)
        self.assertEqual(inst.test[0].description, "Get the history for a known Patient and validate response.")
        self.assertEqual(inst.test[0].id, "01-HistoryPatient")
        self.assertEqual(inst.test[0].name, "History Patient")
        self.assertEqual(inst.text.status, "generated")
        self.assertEqual(inst.url, "http://hl7.org/fhir/TestScript/testscript-example-history")
        self.assertEqual(inst.variable[0].name, "createResourceId")
        self.assertEqual(inst.variable[0].path, "Patient/id")
        self.assertEqual(inst.variable[0].sourceId, "fixture-patient-create")
        self.assertEqual(inst.version, "1.0")
    
    def testTestScript3(self):
        inst = self.instantiate_from("testscript-example-update.json")
        self.assertIsNotNone(inst, "Must have instantiated a TestScript instance")
        self.implTestScript3(inst)
        
        js = inst.as_json()
        self.assertEqual("TestScript", js["resourceType"])
        inst2 = testscript.TestScript(js)
        self.implTestScript3(inst2)
        self.evaluate_simplified_json(inst2)
    
    def implTestScript3(self, inst):
        self.assertEqual(inst.contact[0].name, "Support")
        self.assertEqual(inst.contact[0].telecom[0].system, "email")
        self.assertEqual(inst.contact[0].telecom[0].use, "work")
        self.assertEqual(inst.contact[0].telecom[0].value, "support@HL7.org")
        self.assertEqual(inst.copyright, "© HL7.org 2011+")
        self.assertEqual(inst.date.date, FHIRDate("2017-01-18").date)
        self.assertEqual(inst.date.as_json(), "2017-01-18")
        self.assertEqual(inst.description, "TestScript example resource with setup to delete if present and create a new instance of a Patient; and single test definition to update that Patient with various asserts.")
        self.assertTrue(inst.experimental)
        self.assertFalse(inst.fixture[0].autocreate)
        self.assertFalse(inst.fixture[0].autodelete)
        self.assertEqual(inst.fixture[0].id, "fixture-patient-create")
        self.assertFalse(inst.fixture[1].autocreate)
        self.assertFalse(inst.fixture[1].autodelete)
        self.assertEqual(inst.fixture[1].id, "fixture-patient-update")
        self.assertEqual(inst.id, "testscript-example-update")
        self.assertEqual(inst.identifier.system, "urn:ietf:rfc:3986")
        self.assertEqual(inst.identifier.value, "urn:oid:1.3.6.1.4.1.21367.2005.3.7.9882")
        self.assertEqual(inst.jurisdiction[0].coding[0].code, "US")
        self.assertEqual(inst.jurisdiction[0].coding[0].display, "United States of America (the)")
        self.assertEqual(inst.jurisdiction[0].coding[0].system, "urn:iso:std:iso:3166")
        self.assertEqual(inst.metadata.capability[0].capabilities, "CapabilityStatement/example")
        self.assertEqual(inst.metadata.capability[0].description, "Patient Update and Delete Operations")
        self.assertEqual(inst.metadata.capability[0].link[0], "http://hl7.org/fhir/http.html#update")
        self.assertEqual(inst.metadata.capability[0].link[1], "http://hl7.org/fhir/http.html#delete")
        self.assertTrue(inst.metadata.capability[0].required)
        self.assertFalse(inst.metadata.capability[0].validated)
        self.assertEqual(inst.metadata.link[0].description, "Demographics and other administrative information about an individual or animal receiving care or other health-related services.")
        self.assertEqual(inst.metadata.link[0].url, "http://hl7.org/fhir/patient.html")
        self.assertEqual(inst.name, "TestScript Example Update")
        self.assertEqual(inst.publisher, "HL7")
        self.assertEqual(inst.purpose, "Patient (Conditional) Create, Update, Delete Operations")
        self.assertEqual(inst.setup.action[0].operation.accept, "xml")
        self.assertEqual(inst.setup.action[0].operation.description, "Execute a delete operation to insure the patient does not exist on the server.")
        self.assertTrue(inst.setup.action[0].operation.encodeRequestUrl)
        self.assertEqual(inst.setup.action[0].operation.label, "SetupDeletePatient")
        self.assertEqual(inst.setup.action[0].operation.params, "/${createResourceId}")
        self.assertEqual(inst.setup.action[0].operation.resource, "Patient")
        self.assertEqual(inst.setup.action[0].operation.type.code, "delete")
        self.assertEqual(inst.setup.action[0].operation.type.system, "http://terminology.hl7.org/CodeSystem/testscript-operation-codes")
        self.assertEqual(inst.setup.action[1].assert_fhir.description, "Confirm that the returned HTTP status is 200(OK) or 204(No Content).")
        self.assertEqual(inst.setup.action[1].assert_fhir.direction, "response")
        self.assertEqual(inst.setup.action[1].assert_fhir.operator, "in")
        self.assertEqual(inst.setup.action[1].assert_fhir.responseCode, "200,204")
        self.assertFalse(inst.setup.action[1].assert_fhir.warningOnly)
        self.assertEqual(inst.setup.action[2].operation.accept, "xml")
        self.assertEqual(inst.setup.action[2].operation.contentType, "xml")
        self.assertEqual(inst.setup.action[2].operation.description, "Create patient resource on test server using the contents of fixture-patient-create")
        self.assertTrue(inst.setup.action[2].operation.encodeRequestUrl)
        self.assertEqual(inst.setup.action[2].operation.label, "SetupCreatePatient")
        self.assertEqual(inst.setup.action[2].operation.params, "/${createResourceId}")
        self.assertEqual(inst.setup.action[2].operation.resource, "Patient")
        self.assertEqual(inst.setup.action[2].operation.sourceId, "fixture-patient-create")
        self.assertEqual(inst.setup.action[2].operation.type.code, "update")
        self.assertEqual(inst.setup.action[2].operation.type.system, "http://terminology.hl7.org/CodeSystem/testscript-operation-codes")
        self.assertEqual(inst.setup.action[3].assert_fhir.description, "Confirm that the returned HTTP status is 201(Created).")
        self.assertEqual(inst.setup.action[3].assert_fhir.direction, "response")
        self.assertEqual(inst.setup.action[3].assert_fhir.responseCode, "201")
        self.assertFalse(inst.setup.action[3].assert_fhir.warningOnly)
        self.assertEqual(inst.status, "draft")
        self.assertEqual(inst.test[0].action[0].operation.accept, "xml")
        self.assertEqual(inst.test[0].action[0].operation.contentType, "xml")
        self.assertEqual(inst.test[0].action[0].operation.description, "Update patient resource on test server using the contents of fixture-patient-update")
        self.assertTrue(inst.test[0].action[0].operation.encodeRequestUrl)
        self.assertEqual(inst.test[0].action[0].operation.label, "SetupUpdatePatient")
        self.assertEqual(inst.test[0].action[0].operation.params, "/${createResourceId}")
        self.assertEqual(inst.test[0].action[0].operation.resource, "Patient")
        self.assertEqual(inst.test[0].action[0].operation.sourceId, "fixture-patient-update")
        self.assertEqual(inst.test[0].action[0].operation.type.code, "update")
        self.assertEqual(inst.test[0].action[0].operation.type.system, "http://terminology.hl7.org/CodeSystem/testscript-operation-codes")
        self.assertEqual(inst.test[0].action[1].assert_fhir.description, "Confirm that the returned HTTP status is 200(OK).")
        self.assertEqual(inst.test[0].action[1].assert_fhir.response, "okay")
        self.assertFalse(inst.test[0].action[1].assert_fhir.warningOnly)
        self.assertEqual(inst.test[0].action[2].assert_fhir.contentType, "xml")
        self.assertEqual(inst.test[0].action[2].assert_fhir.description, "Confirm that the returned format is XML.")
        self.assertFalse(inst.test[0].action[2].assert_fhir.warningOnly)
        self.assertEqual(inst.test[0].action[3].assert_fhir.description, "Confirm that the returned HTTP Header Last-Modified is present. Warning only as the server might not support versioning.")
        self.assertEqual(inst.test[0].action[3].assert_fhir.headerField, "Last-Modified")
        self.assertEqual(inst.test[0].action[3].assert_fhir.operator, "notEmpty")
        self.assertTrue(inst.test[0].action[3].assert_fhir.warningOnly)
        self.assertEqual(inst.test[0].description, "Update a Patient and validate response.")
        self.assertEqual(inst.test[0].id, "01-UpdatePatient")
        self.assertEqual(inst.test[0].name, "Update Patient")
        self.assertEqual(inst.text.status, "generated")
        self.assertEqual(inst.url, "http://hl7.org/fhir/TestScript/testscript-example-update")
        self.assertEqual(inst.variable[0].name, "createResourceId")
        self.assertEqual(inst.variable[0].path, "Patient/id")
        self.assertEqual(inst.variable[0].sourceId, "fixture-patient-create")
        self.assertEqual(inst.version, "1.0")
    
    def testTestScript4(self):
        inst = self.instantiate_from("testscript-example-search.json")
        self.assertIsNotNone(inst, "Must have instantiated a TestScript instance")
        self.implTestScript4(inst)
        
        js = inst.as_json()
        self.assertEqual("TestScript", js["resourceType"])
        inst2 = testscript.TestScript(js)
        self.implTestScript4(inst2)
        self.evaluate_simplified_json(inst2)
    
    def implTestScript4(self, inst):
        self.assertEqual(inst.contact[0].name, "Support")
        self.assertEqual(inst.contact[0].telecom[0].system, "email")
        self.assertEqual(inst.contact[0].telecom[0].use, "work")
        self.assertEqual(inst.contact[0].telecom[0].value, "support@HL7.org")
        self.assertEqual(inst.copyright, "© HL7.org 2011+")
        self.assertEqual(inst.date.date, FHIRDate("2017-01-18").date)
        self.assertEqual(inst.date.as_json(), "2017-01-18")
        self.assertEqual(inst.description, "TestScript example resource with simple Patient search test. The read tests will utilize user defined dynamic variables that will hold the Patient search parameter values.")
        self.assertTrue(inst.experimental)
        self.assertFalse(inst.fixture[0].autocreate)
        self.assertFalse(inst.fixture[0].autodelete)
        self.assertEqual(inst.fixture[0].id, "fixture-patient-create")
        self.assertEqual(inst.id, "testscript-example-search")
        self.assertEqual(inst.identifier.system, "urn:ietf:rfc:3986")
        self.assertEqual(inst.identifier.value, "urn:oid:1.3.6.1.4.1.21367.2005.3.7.9881")
        self.assertEqual(inst.jurisdiction[0].coding[0].code, "US")
        self.assertEqual(inst.jurisdiction[0].coding[0].display, "United States of America (the)")
        self.assertEqual(inst.jurisdiction[0].coding[0].system, "urn:iso:std:iso:3166")
        self.assertEqual(inst.metadata.capability[0].capabilities, "CapabilityStatement/example")
        self.assertEqual(inst.metadata.capability[0].description, "Patient Search Operation")
        self.assertEqual(inst.metadata.capability[0].link[0], "http://hl7.org/fhir/http.html#search")
        self.assertTrue(inst.metadata.capability[0].required)
        self.assertFalse(inst.metadata.capability[0].validated)
        self.assertEqual(inst.metadata.link[0].description, "Demographics and other administrative information about an individual or animal receiving care or other health-related services.")
        self.assertEqual(inst.metadata.link[0].url, "http://hl7.org/fhir/patient.html")
        self.assertEqual(inst.name, "TestScript Example Search")
        self.assertEqual(inst.publisher, "HL7")
        self.assertEqual(inst.purpose, "Patient Search Operation")
        self.assertEqual(inst.setup.action[0].operation.accept, "xml")
        self.assertEqual(inst.setup.action[0].operation.description, "Test simple search to verify server support.")
        self.assertTrue(inst.setup.action[0].operation.encodeRequestUrl)
        self.assertEqual(inst.setup.action[0].operation.params, "?family=DONTEXPECTAMATCH&given=DONTEXPECTAMATCH")
        self.assertEqual(inst.setup.action[0].operation.resource, "Patient")
        self.assertEqual(inst.setup.action[0].operation.type.code, "search")
        self.assertEqual(inst.setup.action[0].operation.type.system, "http://terminology.hl7.org/CodeSystem/testscript-operation-codes")
        self.assertEqual(inst.setup.action[1].assert_fhir.description, "Confirm that the request url contains the family search parameter.")
        self.assertEqual(inst.setup.action[1].assert_fhir.direction, "request")
        self.assertEqual(inst.setup.action[1].assert_fhir.operator, "contains")
        self.assertEqual(inst.setup.action[1].assert_fhir.requestURL, "family")
        self.assertFalse(inst.setup.action[1].assert_fhir.warningOnly)
        self.assertEqual(inst.setup.action[2].assert_fhir.description, "Confirm that the returned HTTP status is 200(OK).")
        self.assertEqual(inst.setup.action[2].assert_fhir.direction, "response")
        self.assertEqual(inst.setup.action[2].assert_fhir.responseCode, "200")
        self.assertFalse(inst.setup.action[2].assert_fhir.warningOnly)
        self.assertEqual(inst.setup.action[3].assert_fhir.description, "Confirm that the returned resource type is Bundle.")
        self.assertEqual(inst.setup.action[3].assert_fhir.resource, "Bundle")
        self.assertFalse(inst.setup.action[3].assert_fhir.warningOnly)
        self.assertEqual(inst.setup.action[4].assert_fhir.description, "Confirm that the returned Bundle correctly defines the navigation links.")
        self.assertTrue(inst.setup.action[4].assert_fhir.navigationLinks)
        self.assertFalse(inst.setup.action[4].assert_fhir.warningOnly)
        self.assertEqual(inst.status, "draft")
        self.assertEqual(inst.test[0].action[0].operation.accept, "xml")
        self.assertEqual(inst.test[0].action[0].operation.contentType, "xml")
        self.assertEqual(inst.test[0].action[0].operation.description, "Create a Patient resource and capture the returned HTTP Header Location.")
        self.assertTrue(inst.test[0].action[0].operation.encodeRequestUrl)
        self.assertEqual(inst.test[0].action[0].operation.resource, "Patient")
        self.assertEqual(inst.test[0].action[0].operation.responseId, "PatientCreateResponse")
        self.assertEqual(inst.test[0].action[0].operation.sourceId, "fixture-patient-create")
        self.assertEqual(inst.test[0].action[0].operation.type.code, "create")
        self.assertEqual(inst.test[0].action[0].operation.type.system, "http://terminology.hl7.org/CodeSystem/testscript-operation-codes")
        self.assertEqual(inst.test[0].action[1].assert_fhir.description, "Confirm that the returned HTTP status is 201(Created).")
        self.assertEqual(inst.test[0].action[1].assert_fhir.response, "created")
        self.assertFalse(inst.test[0].action[1].assert_fhir.warningOnly)
        self.assertEqual(inst.test[0].action[2].assert_fhir.description, "Confirm that the returned HTTP Header Location is present.")
        self.assertEqual(inst.test[0].action[2].assert_fhir.direction, "response")
        self.assertEqual(inst.test[0].action[2].assert_fhir.headerField, "Location")
        self.assertEqual(inst.test[0].action[2].assert_fhir.operator, "notEmpty")
        self.assertFalse(inst.test[0].action[2].assert_fhir.warningOnly)
        self.assertEqual(inst.test[0].action[3].operation.accept, "xml")
        self.assertEqual(inst.test[0].action[3].operation.description, "Read the created Patient using the captured Location URL value.")
        self.assertTrue(inst.test[0].action[3].operation.encodeRequestUrl)
        self.assertEqual(inst.test[0].action[3].operation.type.code, "read")
        self.assertEqual(inst.test[0].action[3].operation.type.system, "http://terminology.hl7.org/CodeSystem/testscript-operation-codes")
        self.assertEqual(inst.test[0].action[3].operation.url, "${PatientCreateLocation}")
        self.assertEqual(inst.test[0].action[4].assert_fhir.description, "Confirm that the returned HTTP status is 200(OK).")
        self.assertEqual(inst.test[0].action[4].assert_fhir.response, "okay")
        self.assertFalse(inst.test[0].action[4].assert_fhir.warningOnly)
        self.assertEqual(inst.test[0].action[5].assert_fhir.description, "Confirm that the returned resource type is Patient.")
        self.assertEqual(inst.test[0].action[5].assert_fhir.resource, "Patient")
        self.assertFalse(inst.test[0].action[5].assert_fhir.warningOnly)
        self.assertEqual(inst.test[0].description, "Create a Patient resource and capture the returned HTTP Header Location. Then search for (read) that Patient using the Location URL value and validate the response.")
        self.assertEqual(inst.test[0].id, "01-PatientCreateSearch")
        self.assertEqual(inst.test[0].name, "Patient Create Search")
        self.assertEqual(inst.test[1].action[0].operation.accept, "xml")
        self.assertEqual(inst.test[1].action[0].operation.description, "Search for Patient resources on the destination test system.")
        self.assertTrue(inst.test[1].action[0].operation.encodeRequestUrl)
        self.assertEqual(inst.test[1].action[0].operation.params, "?family=${PatientSearchFamilyName}&given=${PatientSearchGivenName}")
        self.assertEqual(inst.test[1].action[0].operation.resource, "Patient")
        self.assertEqual(inst.test[1].action[0].operation.type.code, "search")
        self.assertEqual(inst.test[1].action[0].operation.type.system, "http://terminology.hl7.org/CodeSystem/testscript-operation-codes")
        self.assertEqual(inst.test[1].action[1].assert_fhir.description, "Confirm that the returned HTTP status is 200(OK).")
        self.assertEqual(inst.test[1].action[1].assert_fhir.response, "okay")
        self.assertFalse(inst.test[1].action[1].assert_fhir.warningOnly)
        self.assertEqual(inst.test[1].action[2].assert_fhir.contentType, "xml")
        self.assertEqual(inst.test[1].action[2].assert_fhir.description, "Confirm that the returned format is XML.")
        self.assertFalse(inst.test[1].action[2].assert_fhir.warningOnly)
        self.assertEqual(inst.test[1].action[3].assert_fhir.description, "Confirm that the returned resource type is Bundle.")
        self.assertEqual(inst.test[1].action[3].assert_fhir.resource, "Bundle")
        self.assertFalse(inst.test[1].action[3].assert_fhir.warningOnly)
        self.assertEqual(inst.test[1].action[4].assert_fhir.description, "Confirm that the returned Bundle conforms to the base FHIR specification.")
        self.assertEqual(inst.test[1].action[4].assert_fhir.validateProfileId, "bundle-profile")
        self.assertFalse(inst.test[1].action[4].assert_fhir.warningOnly)
        self.assertEqual(inst.test[1].action[5].assert_fhir.description, "Confirm that the returned Bundle type equals 'searchset'.")
        self.assertEqual(inst.test[1].action[5].assert_fhir.operator, "equals")
        self.assertEqual(inst.test[1].action[5].assert_fhir.path, "fhir:Bundle/fhir:type/@value")
        self.assertEqual(inst.test[1].action[5].assert_fhir.value, "searchset")
        self.assertFalse(inst.test[1].action[5].assert_fhir.warningOnly)
        self.assertEqual(inst.test[1].action[6].assert_fhir.description, "Confirm that the returned Bundle total is greater than or equal to the number of returned entries.")
        self.assertEqual(inst.test[1].action[6].assert_fhir.expression, "Bundle.total.toInteger() >= entry.count()")
        self.assertFalse(inst.test[1].action[6].assert_fhir.warningOnly)
        self.assertEqual(inst.test[1].description, "Search for Patient resources using the user defined dynamic variables ${PatientSearchFamilyName} and ${PatientSearchGivenName} and validate response.")
        self.assertEqual(inst.test[1].id, "02-PatientSearchDynamic")
        self.assertEqual(inst.test[1].name, "Patient Search Dynamic")
        self.assertEqual(inst.text.status, "generated")
        self.assertEqual(inst.url, "http://hl7.org/fhir/TestScript/testscript-example-search")
        self.assertEqual(inst.variable[0].headerField, "Location")
        self.assertEqual(inst.variable[0].name, "PatientCreateLocation")
        self.assertEqual(inst.variable[0].sourceId, "PatientCreateResponse")
        self.assertEqual(inst.variable[1].description, "Enter patient search criteria for a known family name on the target system")
        self.assertEqual(inst.variable[1].hint, "[Family name]")
        self.assertEqual(inst.variable[1].name, "PatientSearchFamilyName")
        self.assertEqual(inst.variable[2].description, "Enter patient search criteria for a known given name on the target system")
        self.assertEqual(inst.variable[2].hint, "[Given name]")
        self.assertEqual(inst.variable[2].name, "PatientSearchGivenName")
        self.assertEqual(inst.variable[3].description, "Evaluate the returned Patient searchset Bundle.total value")
        self.assertEqual(inst.variable[3].expression, "Bundle.total.toInteger()")
        self.assertEqual(inst.variable[3].name, "PatientSearchBundleTotal")
        self.assertEqual(inst.version, "1.0")
    
    def testTestScript5(self):
        inst = self.instantiate_from("testscript-example.json")
        self.assertIsNotNone(inst, "Must have instantiated a TestScript instance")
        self.implTestScript5(inst)
        
        js = inst.as_json()
        self.assertEqual("TestScript", js["resourceType"])
        inst2 = testscript.TestScript(js)
        self.implTestScript5(inst2)
        self.evaluate_simplified_json(inst2)
    
    def implTestScript5(self, inst):
        self.assertEqual(inst.contact[0].name, "Support")
        self.assertEqual(inst.contact[0].telecom[0].system, "email")
        self.assertEqual(inst.contact[0].telecom[0].use, "work")
        self.assertEqual(inst.contact[0].telecom[0].value, "support@HL7.org")
        self.assertEqual(inst.copyright, "© HL7.org 2011+")
        self.assertEqual(inst.date.date, FHIRDate("2017-01-18").date)
        self.assertEqual(inst.date.as_json(), "2017-01-18")
        self.assertEqual(inst.description, "TestScript example resource with setup to delete if present and create a new instance of a Patient; and single test definition to read the created Patient with various asserts.")
        self.assertTrue(inst.experimental)
        self.assertFalse(inst.fixture[0].autocreate)
        self.assertFalse(inst.fixture[0].autodelete)
        self.assertEqual(inst.fixture[0].id, "fixture-patient-create")
        self.assertFalse(inst.fixture[1].autocreate)
        self.assertFalse(inst.fixture[1].autodelete)
        self.assertEqual(inst.fixture[1].id, "fixture-patient-minimum")
        self.assertEqual(inst.id, "testscript-example")
        self.assertEqual(inst.identifier.system, "urn:ietf:rfc:3986")
        self.assertEqual(inst.identifier.value, "urn:oid:1.3.6.1.4.1.21367.2005.3.7.9876")
        self.assertEqual(inst.jurisdiction[0].coding[0].code, "US")
        self.assertEqual(inst.jurisdiction[0].coding[0].display, "United States of America (the)")
        self.assertEqual(inst.jurisdiction[0].coding[0].system, "urn:iso:std:iso:3166")
        self.assertEqual(inst.metadata.capability[0].capabilities, "CapabilityStatement/example")
        self.assertEqual(inst.metadata.capability[0].description, "Patient Update, Read and Delete Operations")
        self.assertEqual(inst.metadata.capability[0].link[0], "http://hl7.org/fhir/http.html#delete")
        self.assertEqual(inst.metadata.capability[0].link[1], "http://hl7.org/fhir/http.html#read")
        self.assertEqual(inst.metadata.capability[0].link[2], "http://hl7.org/fhir/http.html#update")
        self.assertTrue(inst.metadata.capability[0].required)
        self.assertFalse(inst.metadata.capability[0].validated)
        self.assertEqual(inst.metadata.link[0].description, "Demographics and other administrative information about an individual or animal receiving care or other health-related services.")
        self.assertEqual(inst.metadata.link[0].url, "http://hl7.org/fhir/patient.html")
        self.assertEqual(inst.name, "TestScript Example")
        self.assertEqual(inst.publisher, "HL7")
        self.assertEqual(inst.purpose, "Patient Conditional Create (Update), Read and Delete Operations")
        self.assertEqual(inst.setup.action[0].operation.accept, "json")
        self.assertEqual(inst.setup.action[0].operation.description, "Execute a delete operation to insure the patient does not exist on the server.")
        self.assertTrue(inst.setup.action[0].operation.encodeRequestUrl)
        self.assertEqual(inst.setup.action[0].operation.label, "SetupDeletePatient")
        self.assertEqual(inst.setup.action[0].operation.params, "/${createResourceId}")
        self.assertEqual(inst.setup.action[0].operation.resource, "Patient")
        self.assertEqual(inst.setup.action[0].operation.type.code, "delete")
        self.assertEqual(inst.setup.action[0].operation.type.system, "http://terminology.hl7.org/CodeSystem/testscript-operation-codes")
        self.assertEqual(inst.setup.action[1].assert_fhir.description, "Confirm that the returned HTTP status is 200(OK) or 204(No Content).")
        self.assertEqual(inst.setup.action[1].assert_fhir.direction, "response")
        self.assertEqual(inst.setup.action[1].assert_fhir.operator, "in")
        self.assertEqual(inst.setup.action[1].assert_fhir.responseCode, "200,204")
        self.assertFalse(inst.setup.action[1].assert_fhir.warningOnly)
        self.assertEqual(inst.setup.action[2].operation.accept, "json")
        self.assertEqual(inst.setup.action[2].operation.contentType, "json")
        self.assertEqual(inst.setup.action[2].operation.description, "Create patient resource on test server using the contents of fixture-patient-create")
        self.assertTrue(inst.setup.action[2].operation.encodeRequestUrl)
        self.assertEqual(inst.setup.action[2].operation.label, "SetupCreatePatient")
        self.assertEqual(inst.setup.action[2].operation.params, "/${createResourceId}")
        self.assertEqual(inst.setup.action[2].operation.resource, "Patient")
        self.assertEqual(inst.setup.action[2].operation.sourceId, "fixture-patient-create")
        self.assertEqual(inst.setup.action[2].operation.type.code, "update")
        self.assertEqual(inst.setup.action[2].operation.type.system, "http://terminology.hl7.org/CodeSystem/testscript-operation-codes")
        self.assertEqual(inst.setup.action[3].assert_fhir.description, "Confirm that the returned HTTP status is 201(Created).")
        self.assertEqual(inst.setup.action[3].assert_fhir.direction, "response")
        self.assertEqual(inst.setup.action[3].assert_fhir.responseCode, "201")
        self.assertFalse(inst.setup.action[3].assert_fhir.warningOnly)
        self.assertEqual(inst.setup.action[4].operation.description, "Read the created patient resource on the test server using the id from fixture-patient-create. Verify contents.")
        self.assertTrue(inst.setup.action[4].operation.encodeRequestUrl)
        self.assertEqual(inst.setup.action[4].operation.resource, "Patient")
        self.assertEqual(inst.setup.action[4].operation.targetId, "fixture-patient-create")
        self.assertEqual(inst.setup.action[4].operation.type.code, "read")
        self.assertEqual(inst.setup.action[4].operation.type.system, "http://terminology.hl7.org/CodeSystem/testscript-operation-codes")
        self.assertEqual(inst.setup.action[5].assert_fhir.description, "Confirm that the returned HTTP status is 200(OK).")
        self.assertEqual(inst.setup.action[5].assert_fhir.direction, "response")
        self.assertEqual(inst.setup.action[5].assert_fhir.response, "okay")
        self.assertFalse(inst.setup.action[5].assert_fhir.warningOnly)
        self.assertEqual(inst.setup.action[6].assert_fhir.compareToSourceExpression, "Patient.name.first().family")
        self.assertEqual(inst.setup.action[6].assert_fhir.compareToSourceId, "fixture-patient-create")
        self.assertEqual(inst.setup.action[6].assert_fhir.description, "Confirm that the returned Patient contains the expected family name 'Chalmers'. Uses explicit compareToSourceId reference to fixture-patient-create used to create the Patient.")
        self.assertEqual(inst.setup.action[6].assert_fhir.operator, "equals")
        self.assertFalse(inst.setup.action[6].assert_fhir.warningOnly)
        self.assertEqual(inst.status, "draft")
        self.assertEqual(inst.teardown.action[0].operation.description, "Delete the patient resource on the test server using the id from fixture-patient-create.")
        self.assertTrue(inst.teardown.action[0].operation.encodeRequestUrl)
        self.assertEqual(inst.teardown.action[0].operation.resource, "Patient")
        self.assertEqual(inst.teardown.action[0].operation.targetId, "fixture-patient-create")
        self.assertEqual(inst.teardown.action[0].operation.type.code, "delete")
        self.assertEqual(inst.teardown.action[0].operation.type.system, "http://terminology.hl7.org/CodeSystem/testscript-operation-codes")
        self.assertEqual(inst.test[0].action[0].operation.description, "Read the patient resource on the test server using the id from fixture-patient-create. Prevent URL encoding of the request.")
        self.assertFalse(inst.test[0].action[0].operation.encodeRequestUrl)
        self.assertEqual(inst.test[0].action[0].operation.resource, "Patient")
        self.assertEqual(inst.test[0].action[0].operation.responseId, "fixture-patient-read")
        self.assertEqual(inst.test[0].action[0].operation.targetId, "fixture-patient-create")
        self.assertEqual(inst.test[0].action[0].operation.type.code, "read")
        self.assertEqual(inst.test[0].action[0].operation.type.system, "http://terminology.hl7.org/CodeSystem/testscript-operation-codes")
        self.assertEqual(inst.test[0].action[1].assert_fhir.description, "Confirm that the returned HTTP status is 200(OK).")
        self.assertEqual(inst.test[0].action[1].assert_fhir.direction, "response")
        self.assertEqual(inst.test[0].action[1].assert_fhir.label, "01-ReadPatientOK")
        self.assertEqual(inst.test[0].action[1].assert_fhir.response, "okay")
        self.assertFalse(inst.test[0].action[1].assert_fhir.warningOnly)
        self.assertEqual(inst.test[0].action[2].assert_fhir.description, "Confirm that the returned HTTP Header Last-Modified is present. Warning only as the server might not support versioning.")
        self.assertEqual(inst.test[0].action[2].assert_fhir.direction, "response")
        self.assertEqual(inst.test[0].action[2].assert_fhir.headerField, "Last-Modified")
        self.assertEqual(inst.test[0].action[2].assert_fhir.operator, "notEmpty")
        self.assertTrue(inst.test[0].action[2].assert_fhir.warningOnly)
        self.assertEqual(inst.test[0].action[3].assert_fhir.description, "Confirm that the returned resource type is Patient.")
        self.assertEqual(inst.test[0].action[3].assert_fhir.resource, "Patient")
        self.assertFalse(inst.test[0].action[3].assert_fhir.warningOnly)
        self.assertEqual(inst.test[0].action[4].assert_fhir.description, "Confirm that the returned Patient conforms to the base FHIR specification.")
        self.assertEqual(inst.test[0].action[4].assert_fhir.validateProfileId, "patient-profile")
        self.assertFalse(inst.test[0].action[4].assert_fhir.warningOnly)
        self.assertEqual(inst.test[0].action[5].assert_fhir.description, "Confirm that the returned Patient contains the expected family name 'Chalmers'. Uses explicit sourceId reference to read responseId fixture.")
        self.assertEqual(inst.test[0].action[5].assert_fhir.operator, "equals")
        self.assertEqual(inst.test[0].action[5].assert_fhir.path, "fhir:Patient/fhir:name/fhir:family/@value")
        self.assertEqual(inst.test[0].action[5].assert_fhir.sourceId, "fixture-patient-read")
        self.assertEqual(inst.test[0].action[5].assert_fhir.value, "Chalmers")
        self.assertFalse(inst.test[0].action[5].assert_fhir.warningOnly)
        self.assertEqual(inst.test[0].action[6].assert_fhir.description, "Confirm that the returned Patient contains the expected given name 'Peter'. Uses explicit sourceId reference to read responseId fixture.")
        self.assertEqual(inst.test[0].action[6].assert_fhir.operator, "equals")
        self.assertEqual(inst.test[0].action[6].assert_fhir.path, "fhir:Patient/fhir:name/fhir:given/@value")
        self.assertEqual(inst.test[0].action[6].assert_fhir.sourceId, "fixture-patient-read")
        self.assertEqual(inst.test[0].action[6].assert_fhir.value, "Peter")
        self.assertFalse(inst.test[0].action[6].assert_fhir.warningOnly)
        self.assertEqual(inst.test[0].action[7].assert_fhir.compareToSourceId, "fixture-patient-create")
        self.assertEqual(inst.test[0].action[7].assert_fhir.compareToSourcePath, "fhir:Patient/fhir:name/fhir:family/@value")
        self.assertEqual(inst.test[0].action[7].assert_fhir.operator, "equals")
        self.assertEqual(inst.test[0].action[7].assert_fhir.path, "fhir:Patient/fhir:name/fhir:family/@value")
        self.assertFalse(inst.test[0].action[7].assert_fhir.warningOnly)
        self.assertEqual(inst.test[0].action[8].assert_fhir.compareToSourceId, "fixture-patient-create")
        self.assertEqual(inst.test[0].action[8].assert_fhir.compareToSourcePath, "fhir:Patient/fhir:name/fhir:given/@value")
        self.assertEqual(inst.test[0].action[8].assert_fhir.path, "fhir:Patient/fhir:name/fhir:given/@value")
        self.assertEqual(inst.test[0].action[8].assert_fhir.sourceId, "fixture-patient-read")
        self.assertFalse(inst.test[0].action[8].assert_fhir.warningOnly)
        self.assertEqual(inst.test[0].action[9].assert_fhir.description, "Confirm that the returned resource contains the expected retained elements and values. Warning only to provide users with reviewable results.")
        self.assertEqual(inst.test[0].action[9].assert_fhir.minimumId, "fixture-patient-minimum")
        self.assertTrue(inst.test[0].action[9].assert_fhir.warningOnly)
        self.assertEqual(inst.test[0].description, "Read a Patient and validate response.")
        self.assertEqual(inst.test[0].id, "01-ReadPatient")
        self.assertEqual(inst.test[0].name, "Read Patient")
        self.assertEqual(inst.text.status, "generated")
        self.assertEqual(inst.url, "http://hl7.org/fhir/TestScript/testscript-example")
        self.assertEqual(inst.useContext[0].code.code, "focus")
        self.assertEqual(inst.useContext[0].code.system, "http://terminology.hl7.org/CodeSystem/usage-context-type")
        self.assertEqual(inst.useContext[0].valueCodeableConcept.coding[0].code, "positive")
        self.assertEqual(inst.useContext[0].valueCodeableConcept.coding[0].system, "http://terminology.hl7.org/CodeSystem/variant-state")
        self.assertEqual(inst.variable[0].name, "createResourceId")
        self.assertEqual(inst.variable[0].path, "Patient/id")
        self.assertEqual(inst.variable[0].sourceId, "fixture-patient-create")
        self.assertEqual(inst.version, "1.0")
    
    def testTestScript6(self):
        inst = self.instantiate_from("testscript-example-readtest.json")
        self.assertIsNotNone(inst, "Must have instantiated a TestScript instance")
        self.implTestScript6(inst)
        
        js = inst.as_json()
        self.assertEqual("TestScript", js["resourceType"])
        inst2 = testscript.TestScript(js)
        self.implTestScript6(inst2)
        self.evaluate_simplified_json(inst2)
    
    def implTestScript6(self, inst):
        self.assertEqual(inst.contact[0].name, "Support")
        self.assertEqual(inst.contact[0].telecom[0].system, "email")
        self.assertEqual(inst.contact[0].telecom[0].use, "work")
        self.assertEqual(inst.contact[0].telecom[0].value, "support@HL7.org")
        self.assertEqual(inst.copyright, "© HL7.org 2011+")
        self.assertEqual(inst.date.date, FHIRDate("2017-01-18").date)
        self.assertEqual(inst.date.as_json(), "2017-01-18")
        self.assertEqual(inst.description, "TestScript example resource with ported Sprinkler basic read tests R001, R002, R003, R004. The read tests will utilize user defined dynamic variables that will hold the Patient resource id values.")
        self.assertTrue(inst.experimental)
        self.assertEqual(inst.id, "testscript-example-readtest")
        self.assertEqual(inst.identifier.system, "urn:ietf:rfc:3986")
        self.assertEqual(inst.identifier.value, "urn:oid:1.3.6.1.4.1.21367.2005.3.7.9879")
        self.assertEqual(inst.jurisdiction[0].coding[0].code, "US")
        self.assertEqual(inst.jurisdiction[0].coding[0].display, "United States of America (the)")
        self.assertEqual(inst.jurisdiction[0].coding[0].system, "urn:iso:std:iso:3166")
        self.assertEqual(inst.metadata.capability[0].capabilities, "CapabilityStatement/example")
        self.assertEqual(inst.metadata.capability[0].description, "Patient Read Operation")
        self.assertEqual(inst.metadata.capability[0].link[0], "http://hl7.org/fhir/http.html#read")
        self.assertTrue(inst.metadata.capability[0].required)
        self.assertFalse(inst.metadata.capability[0].validated)
        self.assertEqual(inst.metadata.link[0].description, "Demographics and other administrative information about an individual or animal receiving care or other health-related services.")
        self.assertEqual(inst.metadata.link[0].url, "http://hl7.org/fhir/patient.html")
        self.assertEqual(inst.name, "TestScript Example Read Test")
        self.assertEqual(inst.publisher, "HL7")
        self.assertEqual(inst.purpose, "Patient Read Operation")
        self.assertEqual(inst.status, "draft")
        self.assertEqual(inst.test[0].action[0].operation.accept, "xml")
        self.assertEqual(inst.test[0].action[0].operation.description, "Read the known Patient resource on the destination test system using the user defined dynamic variable ${KnownPatientResourceId}.")
        self.assertTrue(inst.test[0].action[0].operation.encodeRequestUrl)
        self.assertEqual(inst.test[0].action[0].operation.params, "/${KnownPatientResourceId}")
        self.assertEqual(inst.test[0].action[0].operation.resource, "Patient")
        self.assertEqual(inst.test[0].action[0].operation.type.code, "read")
        self.assertEqual(inst.test[0].action[0].operation.type.system, "http://terminology.hl7.org/CodeSystem/testscript-operation-codes")
        self.assertEqual(inst.test[0].action[1].assert_fhir.description, "Confirm that the returned HTTP status is 200(OK).")
        self.assertEqual(inst.test[0].action[1].assert_fhir.response, "okay")
        self.assertFalse(inst.test[0].action[1].assert_fhir.warningOnly)
        self.assertEqual(inst.test[0].action[2].assert_fhir.contentType, "xml")
        self.assertEqual(inst.test[0].action[2].assert_fhir.description, "Confirm that the returned format is XML.")
        self.assertFalse(inst.test[0].action[2].assert_fhir.warningOnly)
        self.assertEqual(inst.test[0].action[3].assert_fhir.description, "Confirm that the returned HTTP Header Last-Modified is present. Warning only as the server might not support versioning.")
        self.assertEqual(inst.test[0].action[3].assert_fhir.headerField, "Last-Modified")
        self.assertEqual(inst.test[0].action[3].assert_fhir.operator, "notEmpty")
        self.assertTrue(inst.test[0].action[3].assert_fhir.warningOnly)
        self.assertEqual(inst.test[0].action[4].assert_fhir.description, "Confirm that the returned resource type is Patient.")
        self.assertEqual(inst.test[0].action[4].assert_fhir.resource, "Patient")
        self.assertFalse(inst.test[0].action[4].assert_fhir.warningOnly)
        self.assertEqual(inst.test[0].action[5].assert_fhir.description, "Confirm that the returned Patient conforms to the base FHIR specification.")
        self.assertEqual(inst.test[0].action[5].assert_fhir.validateProfileId, "patient-profile")
        self.assertFalse(inst.test[0].action[5].assert_fhir.warningOnly)
        self.assertEqual(inst.test[0].description, "Read a known Patient and validate response.")
        self.assertEqual(inst.test[0].id, "R001")
        self.assertEqual(inst.test[0].name, "Sprinkler Read Test R001")
        self.assertEqual(inst.test[1].action[0].operation.accept, "xml")
        self.assertTrue(inst.test[1].action[0].operation.encodeRequestUrl)
        self.assertEqual(inst.test[1].action[0].operation.params, "/1")
        self.assertEqual(inst.test[1].action[0].operation.resource, "Patient")
        self.assertEqual(inst.test[1].action[0].operation.type.code, "read")
        self.assertEqual(inst.test[1].action[0].operation.type.system, "http://terminology.hl7.org/CodeSystem/testscript-operation-codes")
        self.assertEqual(inst.test[1].action[1].assert_fhir.description, "Confirm that the returned HTTP status is 404(Not Found).")
        self.assertEqual(inst.test[1].action[1].assert_fhir.response, "notFound")
        self.assertFalse(inst.test[1].action[1].assert_fhir.warningOnly)
        self.assertEqual(inst.test[1].description, "Read an unknown Resource Type and validate response.")
        self.assertEqual(inst.test[1].id, "R002")
        self.assertEqual(inst.test[1].name, "Sprinkler Read Test R002")
        self.assertEqual(inst.test[2].action[0].operation.accept, "xml")
        self.assertEqual(inst.test[2].action[0].operation.description, "Attempt to read the non-existing Patient resource on the destination test system using the user defined dynamic variable ${NonExistsPatientResourceId}.")
        self.assertTrue(inst.test[2].action[0].operation.encodeRequestUrl)
        self.assertEqual(inst.test[2].action[0].operation.params, "/${NonExistsPatientResourceId}")
        self.assertEqual(inst.test[2].action[0].operation.resource, "Patient")
        self.assertEqual(inst.test[2].action[0].operation.type.code, "read")
        self.assertEqual(inst.test[2].action[0].operation.type.system, "http://terminology.hl7.org/CodeSystem/testscript-operation-codes")
        self.assertEqual(inst.test[2].action[1].assert_fhir.description, "Confirm that the returned HTTP status is 404(Not Found).")
        self.assertEqual(inst.test[2].action[1].assert_fhir.response, "notFound")
        self.assertFalse(inst.test[2].action[1].assert_fhir.warningOnly)
        self.assertEqual(inst.test[2].description, "Read a known, non-existing Patient and validate response.")
        self.assertEqual(inst.test[2].id, "R003")
        self.assertEqual(inst.test[2].name, "Sprinkler Read Test R003")
        self.assertEqual(inst.test[3].action[0].operation.accept, "xml")
        self.assertEqual(inst.test[3].action[0].operation.description, "Attempt to read a Patient resource on the destination test system using known bad formatted resource id.")
        self.assertTrue(inst.test[3].action[0].operation.encodeRequestUrl)
        self.assertEqual(inst.test[3].action[0].operation.params, "/ID-may-not-contain-CAPITALS")
        self.assertEqual(inst.test[3].action[0].operation.resource, "Patient")
        self.assertEqual(inst.test[3].action[0].operation.type.code, "read")
        self.assertEqual(inst.test[3].action[0].operation.type.system, "http://terminology.hl7.org/CodeSystem/testscript-operation-codes")
        self.assertEqual(inst.test[3].action[1].assert_fhir.description, "Confirm that the returned HTTP status is 400(Bad Request).")
        self.assertEqual(inst.test[3].action[1].assert_fhir.response, "bad")
        self.assertFalse(inst.test[3].action[1].assert_fhir.warningOnly)
        self.assertEqual(inst.test[3].description, "Read a Patient using a known bad formatted resource id and validate response.")
        self.assertEqual(inst.test[3].id, "R004")
        self.assertEqual(inst.test[3].name, "Sprinkler Read Test R004")
        self.assertEqual(inst.text.status, "generated")
        self.assertEqual(inst.url, "http://hl7.org/fhir/TestScript/testscript-example-readtest")
        self.assertEqual(inst.variable[0].defaultValue, "example")
        self.assertEqual(inst.variable[0].name, "KnownPatientResourceId")
        self.assertEqual(inst.variable[1].defaultValue, "does-not-exist")
        self.assertEqual(inst.variable[1].name, "NonExistsPatientResourceId")
        self.assertEqual(inst.version, "1.0")

    def evaluate_simplified_json(self, inst):
        """Ensure simplified json."""
        simplified_js, simplified_schema = inst.as_simplified_json()
        self.assertIsNotNone(simplified_js, "Must create simplified json")

        # test values
        for simplified_key, simplified_values in simplified_js.items():
            if not simplified_values:
                continue
            if not isinstance(simplified_values, typing.List):
                simplified_values = [simplified_values]
            for simplified_value in simplified_values:
                simplified_value_is_fhir_resource = 'fhirclient.models' in simplified_value.__class__.__name__
                if simplified_value_is_fhir_resource:
                    msg = "Should simplify value {} {} {}".format(simplified_key, simplified_value.__class__.__name__, vars(simplified_value))
                    self.assertFalse(simplified_value_is_fhir_resource, msg)



        # test simplify identifiers
        if hasattr(inst, 'identifier'):
            assert 'identifier' not in simplified_js
            if inst.identifier:
                simplified_identifiers = [k for k in simplified_js.keys() if k.startswith('identifier_')]
                if isinstance(inst.identifier, typing.List):
                    identifiers_with_values = [i for i in inst.identifier if i.value]
                else:
                    identifiers_with_values = [inst.identifier]
                self.assertEqual(len(identifiers_with_values), len(simplified_identifiers), "Should simplify identifiers.")

        # test simplify lists
        for name in vars(inst):

            if name == 'identifier':
                continue

            if name == 'extension':
                continue

            value = getattr(inst, name)

            is_coding = value.__class__.__name__ == 'Coding' or (isinstance(value, typing.List) and len(value) == 1 and value[0].__class__.__name__ == 'Coding')
            if is_coding:
                # why are we skipping Coding test?
                continue

            is_date = 'FHIRDate' in value.__class__.__name__
            if is_date:
                simplified_value_is_date = 'FHIRDate' in simplified_js[name].__class__.__name__
                self.assertFalse(simplified_value_is_date, "Should simplify Date {} {} {}".format(name, value.__class__.__name__, vars(value)) )

            if isinstance(getattr(inst, name), typing.List) and len(getattr(inst, name)) == 1:
                # Properties that need to be renamed because of language keyword conflicts
                # see mapping
                if name not in simplified_js:
                    name = name.replace("_fhir", "")
                self.assertFalse(isinstance(simplified_js[name], typing.List), "Should simplify lists {}".format(name))

        # test simplify coding
        # meta has known coding attribute 'tags'
        if hasattr(inst, 'meta'):
            if inst.meta and inst.meta.tag and len(inst.meta.tag) > 0:
                simplified_tags = [k for k in simplified_js['meta'].keys() if k.startswith('tag_')]
                self.assertEqual(len(inst.meta.tag), len(simplified_tags), "Should simplify meta tags.")
                self.assertTrue('tag' not in simplified_js['meta'], "Should not have meta.tag")

        # test simplify extensions
        if hasattr(inst, 'extension'):
            if inst.extension and len(inst.extension) > 0:
                assert 'extension' not in simplified_js
                simplified_extensions = [k for k in simplified_js.keys() if k.startswith('extension_')]
                self.assertEqual(len(inst.extension), len(simplified_extensions), "Should simplify extensions.")
                for simplified_extension in simplified_extensions:
                    assert simplified_js[simplified_extension], f"Missing value for {simplified_extension}"
                    assert 'fhirclient.models.coding.Coding' not in str(simplified_js[simplified_extension]), "Should simplify codes"
                    if simplified_js[simplified_extension] == 'NA':
                        logging.getLogger(__name__).warning(
                            "Extension.value is NA for resource_type:{} simplified_extension:{}".format(
                                inst.resource_type, simplified_extension))
        # test simplify schema
        for k in simplified_js:
            assert k in simplified_schema, "Should have a schema definition for {}".format(k)

        # test simplified, flattened
        from flatten_json import flatten
        flattened = flatten(simplified_js, separator='|')
        for flattened_key in flattened:
            dict_ = simplified_schema
            for flattened_key_part in flattened_key.split('|'):
                if flattened_key_part not in dict_ and flattened_key_part.isnumeric():
                    # traverse over list index
                    continue
                dict_ = dict_[flattened_key_part]
                self.assertIsNotNone(dict_, "Should have a schema entry for {}".format(flattened_key_part))
                if 'docstring' not in dict_:
                    logging.getLogger(__name__).warning(
                        "Missing docstring for resource_type:{} flattened_key:{} flattened_key_part:{} dict:{}".format(
                            inst.resource_type, flattened_key, flattened_key_part, dict_))
                    break


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 4.0.1-9346c8cc45 (http://hl7.org/fhir/StructureDefinition/CarePlan) on 2022-06-22.
#  2022, SMART Health IT.


from . import domainresource

class CarePlan(domainresource.DomainResource):
    """ Healthcare plan for patient or group.
    
    Describes the intention of how one or more practitioners intend to deliver
    care for a particular patient, group or community for a period of time,
    possibly limited to care for a specific condition or set of conditions.
    """
    
    resource_type = "CarePlan"

    _attribute_docstrings = {}
    """ Dictionary of attribute documentation."""
    _attribute_docstrings['identifier'] = """External Ids for this plan."""
    _attribute_docstrings['instantiatesCanonical'] = """Instantiates FHIR protocol or definition."""
    _attribute_docstrings['instantiatesUri'] = """Instantiates external protocol or definition."""
    _attribute_docstrings['basedOn'] = """Fulfills CarePlan."""
    _attribute_docstrings['replaces'] = """CarePlan replaced by this CarePlan."""
    _attribute_docstrings['partOf'] = """Part of referenced CarePlan."""
    _attribute_docstrings['status'] = """Indicates whether the plan is currently being acted upon, represents future intentions or is now a historical record."""
    _attribute_docstrings['intent'] = """Indicates the level of authority/intentionality associated with the care plan and where the care plan fits into the workflow chain."""
    _attribute_docstrings['category'] = """Type of plan."""
    _attribute_docstrings['title'] = """Human-friendly name for the care plan."""
    _attribute_docstrings['description'] = """Summary of nature of plan."""
    _attribute_docstrings['subject'] = """Who the care plan is for."""
    _attribute_docstrings['encounter'] = """Encounter created as part of."""
    _attribute_docstrings['period'] = """Time period plan covers."""
    _attribute_docstrings['created'] = """Date record was first recorded."""
    _attribute_docstrings['author'] = """Who is the designated responsible party."""
    _attribute_docstrings['contributor'] = """Who provided the content of the care plan."""
    _attribute_docstrings['careTeam'] = """Who's involved in plan?."""
    _attribute_docstrings['addresses'] = """Health issues this plan addresses."""
    _attribute_docstrings['supportingInfo'] = """Information considered as part of plan."""
    _attribute_docstrings['goal'] = """Desired outcome of plan."""
    _attribute_docstrings['activity'] = """Action to occur as part of plan."""
    _attribute_docstrings['note'] = """Comments about the plan."""

    @classmethod
    def attribute_docstrings(cls):
        """Get dict of attributes docstrings."""
        return cls._attribute_docstrings

    _attribute_enums = {}
    """ Dictionary of enum configuration."""
    _attribute_enums['status'] = {
        'url': 'http://hl7.org/fhir/request-status',
        'restricted_to': ['draft', 'active', 'on-hold', 'revoked', 'completed', 'entered-in-error', 'unknown'],
        'binding_strength': 'required',
        'class_name': 'str'
    }
    _attribute_enums['intent'] = {
        'url': 'http://hl7.org/fhir/request-intent',
        'restricted_to': ['proposal', 'plan', 'directive', 'order', 'original-order', 'reflex-order', 'filler-order', 'instance-order', 'option'],
        'binding_strength': 'required',
        'class_name': 'str'
    }

    @classmethod
    def attribute_enums(cls):
        """Get dict of attributes with enums, Code or CodeableConcept."""
        return cls._attribute_enums

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.identifier = None
        """ External Ids for this plan.
        List of `Identifier` items (represented as `dict` in JSON). """
        
        self.instantiatesCanonical = None
        """ Instantiates FHIR protocol or definition.
        List of `str` items. """
        
        self.instantiatesUri = None
        """ Instantiates external protocol or definition.
        List of `str` items. """
        
        self.basedOn = None
        """ Fulfills CarePlan.
        List of `FHIRReference` items (represented as `dict` in JSON). """
        
        self.replaces = None
        """ CarePlan replaced by this CarePlan.
        List of `FHIRReference` items (represented as `dict` in JSON). """
        
        self.partOf = None
        """ Part of referenced CarePlan.
        List of `FHIRReference` items (represented as `dict` in JSON). """
        
        self.status = None
        """ Indicates whether the plan is currently being acted upon,
        represents future intentions or is now a historical record.
        Type `str`. """
        
        self.intent = None
        """ Indicates the level of authority/intentionality associated with the
        care plan and where the care plan fits into the workflow chain.
        Type `str`. """
        
        self.category = None
        """ Type of plan.
        List of `CodeableConcept` items (represented as `dict` in JSON). """
        
        self.title = None
        """ Human-friendly name for the care plan.
        Type `str`. """
        
        self.description = None
        """ Summary of nature of plan.
        Type `str`. """
        
        self.subject = None
        """ Who the care plan is for.
        Type `FHIRReference` (represented as `dict` in JSON). """
        
        self.encounter = None
        """ Encounter created as part of.
        Type `FHIRReference` (represented as `dict` in JSON). """
        
        self.period = None
        """ Time period plan covers.
        Type `Period` (represented as `dict` in JSON). """
        
        self.created = None
        """ Date record was first recorded.
        Type `FHIRDate` (represented as `str` in JSON). """
        
        self.author = None
        """ Who is the designated responsible party.
        Type `FHIRReference` (represented as `dict` in JSON). """
        
        self.contributor = None
        """ Who provided the content of the care plan.
        List of `FHIRReference` items (represented as `dict` in JSON). """
        
        self.careTeam = None
        """ Who's involved in plan?.
        List of `FHIRReference` items (represented as `dict` in JSON). """
        
        self.addresses = None
        """ Health issues this plan addresses.
        List of `FHIRReference` items (represented as `dict` in JSON). """
        
        self.supportingInfo = None
        """ Information considered as part of plan.
        List of `FHIRReference` items (represented as `dict` in JSON). """
        
        self.goal = None
        """ Desired outcome of plan.
        List of `FHIRReference` items (represented as `dict` in JSON). """
        
        self.activity = None
        """ Action to occur as part of plan.
        List of `CarePlanActivity` items (represented as `dict` in JSON). """
        
        self.note = None
        """ Comments about the plan.
        List of `Annotation` items (represented as `dict` in JSON). """
        
        super(CarePlan, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(CarePlan, self).elementProperties()
        js.extend([
            ("identifier", "identifier", identifier.Identifier, True, None, False),
            ("instantiatesCanonical", "instantiatesCanonical", str, True, None, False),
            ("instantiatesUri", "instantiatesUri", str, True, None, False),
            ("basedOn", "basedOn", fhirreference.FHIRReference, True, None, False),
            ("replaces", "replaces", fhirreference.FHIRReference, True, None, False),
            ("partOf", "partOf", fhirreference.FHIRReference, True, None, False),
            ("status", "status", str, False, None, True),
            ("intent", "intent", str, False, None, True),
            ("category", "category", codeableconcept.CodeableConcept, True, None, False),
            ("title", "title", str, False, None, False),
            ("description", "description", str, False, None, False),
            ("subject", "subject", fhirreference.FHIRReference, False, None, True),
            ("encounter", "encounter", fhirreference.FHIRReference, False, None, False),
            ("period", "period", period.Period, False, None, False),
            ("created", "created", fhirdate.FHIRDate, False, None, False),
            ("author", "author", fhirreference.FHIRReference, False, None, False),
            ("contributor", "contributor", fhirreference.FHIRReference, True, None, False),
            ("careTeam", "careTeam", fhirreference.FHIRReference, True, None, False),
            ("addresses", "addresses", fhirreference.FHIRReference, True, None, False),
            ("supportingInfo", "supportingInfo", fhirreference.FHIRReference, True, None, False),
            ("goal", "goal", fhirreference.FHIRReference, True, None, False),
            ("activity", "activity", CarePlanActivity, True, None, False),
            ("note", "note", annotation.Annotation, True, None, False),
        ])
        return js


from . import backboneelement

class CarePlanActivity(backboneelement.BackboneElement):
    """ Action to occur as part of plan.
    
    Identifies a planned action to occur as part of the plan.  For example, a
    medication to be used, lab tests to perform, self-monitoring, education,
    etc.
    """

    _attribute_docstrings = {}
    """ Dictionary of attribute documentation."""
    _attribute_docstrings['outcomeCodeableConcept'] = """Results of the activity."""
    _attribute_docstrings['outcomeReference'] = """Appointment, Encounter, Procedure, etc.."""
    _attribute_docstrings['progress'] = """Comments about the activity status/progress."""
    _attribute_docstrings['reference'] = """Activity details defined in specific resource."""
    _attribute_docstrings['detail'] = """In-line definition of activity."""

    @classmethod
    def attribute_docstrings(cls):
        """Get dict of attributes docstrings."""
        return cls._attribute_docstrings

    _attribute_enums = {}
    """ Dictionary of enum configuration."""

    @classmethod
    def attribute_enums(cls):
        """Get dict of attributes with enums, Code or CodeableConcept."""
        return cls._attribute_enums

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.outcomeCodeableConcept = None
        """ Results of the activity.
        List of `CodeableConcept` items (represented as `dict` in JSON). """
        
        self.outcomeReference = None
        """ Appointment, Encounter, Procedure, etc.
        List of `FHIRReference` items (represented as `dict` in JSON). """
        
        self.progress = None
        """ Comments about the activity status/progress.
        List of `Annotation` items (represented as `dict` in JSON). """
        
        self.reference = None
        """ Activity details defined in specific resource.
        Type `FHIRReference` (represented as `dict` in JSON). """
        
        self.detail = None
        """ In-line definition of activity.
        Type `CarePlanActivityDetail` (represented as `dict` in JSON). """
        
        super(CarePlanActivity, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(CarePlanActivity, self).elementProperties()
        js.extend([
            ("outcomeCodeableConcept", "outcomeCodeableConcept", codeableconcept.CodeableConcept, True, None, False),
            ("outcomeReference", "outcomeReference", fhirreference.FHIRReference, True, None, False),
            ("progress", "progress", annotation.Annotation, True, None, False),
            ("reference", "reference", fhirreference.FHIRReference, False, None, False),
            ("detail", "detail", CarePlanActivityDetail, False, None, False),
        ])
        return js


class CarePlanActivityDetail(backboneelement.BackboneElement):
    """ In-line definition of activity.
    
    A simple summary of a planned activity suitable for a general care plan
    system (e.g. form driven) that doesn't know about specific resources such
    as procedure etc.
    """

    _attribute_docstrings = {}
    """ Dictionary of attribute documentation."""
    _attribute_docstrings['kind'] = """A description of the kind of resource the in-line definition of a care plan activity is representing.  The CarePlan.activity.detail is an in-line definition when a resource is not referenced using CarePlan.activity.reference.  For example, a MedicationRequest, a ServiceRequest, or a CommunicationRequest."""
    _attribute_docstrings['instantiatesCanonical'] = """Instantiates FHIR protocol or definition."""
    _attribute_docstrings['instantiatesUri'] = """Instantiates external protocol or definition."""
    _attribute_docstrings['code'] = """Detail type of activity."""
    _attribute_docstrings['reasonCode'] = """Why activity should be done or why activity was prohibited."""
    _attribute_docstrings['reasonReference'] = """Why activity is needed."""
    _attribute_docstrings['goal'] = """Goals this activity relates to."""
    _attribute_docstrings['status'] = """Identifies what progress is being made for the specific activity."""
    _attribute_docstrings['statusReason'] = """Reason for current status."""
    _attribute_docstrings['doNotPerform'] = """If true, activity is prohibiting action."""
    _attribute_docstrings['scheduledTiming'] = """When activity is to occur."""
    _attribute_docstrings['scheduledPeriod'] = """When activity is to occur."""
    _attribute_docstrings['scheduledString'] = """When activity is to occur."""
    _attribute_docstrings['location'] = """Where it should happen."""
    _attribute_docstrings['performer'] = """Who will be responsible?."""
    _attribute_docstrings['productCodeableConcept'] = """What is to be administered/supplied."""
    _attribute_docstrings['productReference'] = """What is to be administered/supplied."""
    _attribute_docstrings['dailyAmount'] = """How to consume/day?."""
    _attribute_docstrings['quantity'] = """How much to administer/supply/consume."""
    _attribute_docstrings['description'] = """Extra info describing activity to perform."""

    @classmethod
    def attribute_docstrings(cls):
        """Get dict of attributes docstrings."""
        return cls._attribute_docstrings

    _attribute_enums = {}
    """ Dictionary of enum configuration."""
    _attribute_enums['kind'] = {
        'url': 'http://hl7.org/fhir/resource-types',
        'restricted_to': ['Account', 'ActivityDefinition', 'AdverseEvent', 'AllergyIntolerance', 'Appointment', 'AppointmentResponse', 'AuditEvent', 'Basic', 'Binary', 'BiologicallyDerivedProduct', 'BodyStructure', 'Bundle', 'CapabilityStatement', 'CarePlan', 'CareTeam', 'CatalogEntry', 'ChargeItem', 'ChargeItemDefinition', 'Claim', 'ClaimResponse', 'ClinicalImpression', 'CodeSystem', 'Communication', 'CommunicationRequest', 'CompartmentDefinition', 'Composition', 'ConceptMap', 'Condition', 'Consent', 'Contract', 'Coverage', 'CoverageEligibilityRequest', 'CoverageEligibilityResponse', 'DetectedIssue', 'Device', 'DeviceDefinition', 'DeviceMetric', 'DeviceRequest', 'DeviceUseStatement', 'DiagnosticReport', 'DocumentManifest', 'DocumentReference', 'DomainResource', 'EffectEvidenceSynthesis', 'Encounter', 'Endpoint', 'EnrollmentRequest', 'EnrollmentResponse', 'EpisodeOfCare', 'EventDefinition', 'Evidence', 'EvidenceVariable', 'ExampleScenario', 'ExplanationOfBenefit', 'FamilyMemberHistory', 'Flag', 'Goal', 'GraphDefinition', 'Group', 'GuidanceResponse', 'HealthcareService', 'ImagingStudy', 'Immunization', 'ImmunizationEvaluation', 'ImmunizationRecommendation', 'ImplementationGuide', 'InsurancePlan', 'Invoice', 'Library', 'Linkage', 'List', 'Location', 'Measure', 'MeasureReport', 'Media', 'Medication', 'MedicationAdministration', 'MedicationDispense', 'MedicationKnowledge', 'MedicationRequest', 'MedicationStatement', 'MedicinalProduct', 'MedicinalProductAuthorization', 'MedicinalProductContraindication', 'MedicinalProductIndication', 'MedicinalProductIngredient', 'MedicinalProductInteraction', 'MedicinalProductManufactured', 'MedicinalProductPackaged', 'MedicinalProductPharmaceutical', 'MedicinalProductUndesirableEffect', 'MessageDefinition', 'MessageHeader', 'MolecularSequence', 'NamingSystem', 'NutritionOrder', 'Observation', 'ObservationDefinition', 'OperationDefinition', 'OperationOutcome', 'Organization', 'OrganizationAffiliation', 'Parameters', 'Patient', 'PaymentNotice', 'PaymentReconciliation', 'Person', 'PlanDefinition', 'Practitioner', 'PractitionerRole', 'Procedure', 'Provenance', 'Questionnaire', 'QuestionnaireResponse', 'RelatedPerson', 'RequestGroup', 'ResearchDefinition', 'ResearchElementDefinition', 'ResearchStudy', 'ResearchSubject', 'Resource', 'RiskAssessment', 'RiskEvidenceSynthesis', 'Schedule', 'SearchParameter', 'ServiceRequest', 'Slot', 'Specimen', 'SpecimenDefinition', 'StructureDefinition', 'StructureMap', 'Subscription', 'Substance', 'SubstanceNucleicAcid', 'SubstancePolymer', 'SubstanceProtein', 'SubstanceReferenceInformation', 'SubstanceSourceMaterial', 'SubstanceSpecification', 'SupplyDelivery', 'SupplyRequest', 'Task', 'TerminologyCapabilities', 'TestReport', 'TestScript', 'ValueSet', 'VerificationResult', 'VisionPrescription'],
        'binding_strength': 'required',
        'class_name': 'str'
    }
    _attribute_enums['status'] = {
        'url': 'http://hl7.org/fhir/care-plan-activity-status',
        'restricted_to': ['not-started', 'scheduled', 'in-progress', 'on-hold', 'completed', 'cancelled', 'stopped', 'unknown', 'entered-in-error'],
        'binding_strength': 'required',
        'class_name': 'str'
    }

    @classmethod
    def attribute_enums(cls):
        """Get dict of attributes with enums, Code or CodeableConcept."""
        return cls._attribute_enums

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.kind = None
        """ A description of the kind of resource the in-line definition of a
        care plan activity is representing.  The CarePlan.activity.detail
        is an in-line definition when a resource is not referenced using
        CarePlan.activity.reference.  For example, a MedicationRequest, a
        ServiceRequest, or a CommunicationRequest.
        Type `str`. """
        
        self.instantiatesCanonical = None
        """ Instantiates FHIR protocol or definition.
        List of `str` items. """
        
        self.instantiatesUri = None
        """ Instantiates external protocol or definition.
        List of `str` items. """
        
        self.code = None
        """ Detail type of activity.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.reasonCode = None
        """ Why activity should be done or why activity was prohibited.
        List of `CodeableConcept` items (represented as `dict` in JSON). """
        
        self.reasonReference = None
        """ Why activity is needed.
        List of `FHIRReference` items (represented as `dict` in JSON). """
        
        self.goal = None
        """ Goals this activity relates to.
        List of `FHIRReference` items (represented as `dict` in JSON). """
        
        self.status = None
        """ Identifies what progress is being made for the specific activity.
        Type `str`. """
        
        self.statusReason = None
        """ Reason for current status.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.doNotPerform = None
        """ If true, activity is prohibiting action.
        Type `bool`. """
        
        self.scheduledTiming = None
        """ When activity is to occur.
        Type `Timing` (represented as `dict` in JSON). """
        
        self.scheduledPeriod = None
        """ When activity is to occur.
        Type `Period` (represented as `dict` in JSON). """
        
        self.scheduledString = None
        """ When activity is to occur.
        Type `str`. """
        
        self.location = None
        """ Where it should happen.
        Type `FHIRReference` (represented as `dict` in JSON). """
        
        self.performer = None
        """ Who will be responsible?.
        List of `FHIRReference` items (represented as `dict` in JSON). """
        
        self.productCodeableConcept = None
        """ What is to be administered/supplied.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.productReference = None
        """ What is to be administered/supplied.
        Type `FHIRReference` (represented as `dict` in JSON). """
        
        self.dailyAmount = None
        """ How to consume/day?.
        Type `Quantity` (represented as `dict` in JSON). """
        
        self.quantity = None
        """ How much to administer/supply/consume.
        Type `Quantity` (represented as `dict` in JSON). """
        
        self.description = None
        """ Extra info describing activity to perform.
        Type `str`. """
        
        super(CarePlanActivityDetail, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(CarePlanActivityDetail, self).elementProperties()
        js.extend([
            ("kind", "kind", str, False, None, False),
            ("instantiatesCanonical", "instantiatesCanonical", str, True, None, False),
            ("instantiatesUri", "instantiatesUri", str, True, None, False),
            ("code", "code", codeableconcept.CodeableConcept, False, None, False),
            ("reasonCode", "reasonCode", codeableconcept.CodeableConcept, True, None, False),
            ("reasonReference", "reasonReference", fhirreference.FHIRReference, True, None, False),
            ("goal", "goal", fhirreference.FHIRReference, True, None, False),
            ("status", "status", str, False, None, True),
            ("statusReason", "statusReason", codeableconcept.CodeableConcept, False, None, False),
            ("doNotPerform", "doNotPerform", bool, False, None, False),
            ("scheduledTiming", "scheduledTiming", timing.Timing, False, "scheduled", False),
            ("scheduledPeriod", "scheduledPeriod", period.Period, False, "scheduled", False),
            ("scheduledString", "scheduledString", str, False, "scheduled", False),
            ("location", "location", fhirreference.FHIRReference, False, None, False),
            ("performer", "performer", fhirreference.FHIRReference, True, None, False),
            ("productCodeableConcept", "productCodeableConcept", codeableconcept.CodeableConcept, False, "product", False),
            ("productReference", "productReference", fhirreference.FHIRReference, False, "product", False),
            ("dailyAmount", "dailyAmount", quantity.Quantity, False, None, False),
            ("quantity", "quantity", quantity.Quantity, False, None, False),
            ("description", "description", str, False, None, False),
        ])
        return js


import sys
try:
    from . import annotation
except ImportError:
    annotation = sys.modules[__package__ + '.annotation']
try:
    from . import codeableconcept
except ImportError:
    codeableconcept = sys.modules[__package__ + '.codeableconcept']
try:
    from . import fhirdate
except ImportError:
    fhirdate = sys.modules[__package__ + '.fhirdate']
try:
    from . import fhirreference
except ImportError:
    fhirreference = sys.modules[__package__ + '.fhirreference']
try:
    from . import identifier
except ImportError:
    identifier = sys.modules[__package__ + '.identifier']
try:
    from . import period
except ImportError:
    period = sys.modules[__package__ + '.period']
try:
    from . import quantity
except ImportError:
    quantity = sys.modules[__package__ + '.quantity']
try:
    from . import timing
except ImportError:
    timing = sys.modules[__package__ + '.timing']



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 4.0.1-9346c8cc45 (http://hl7.org/fhir/StructureDefinition/DetectedIssue) on 2022-06-22.
#  2022, SMART Health IT.


from . import domainresource

class DetectedIssue(domainresource.DomainResource):
    """ Clinical issue with action.
    
    Indicates an actual or potential clinical issue with or between one or more
    active or proposed clinical actions for a patient; e.g. Drug-drug
    interaction, Ineffective treatment frequency, Procedure-condition conflict,
    etc.
    """
    
    resource_type = "DetectedIssue"

    _attribute_docstrings = {}
    """ Dictionary of attribute documentation."""
    _attribute_docstrings['identifier'] = """Unique id for the detected issue."""
    _attribute_docstrings['status'] = """Indicates the status of the detected issue."""
    _attribute_docstrings['code'] = """Issue Category, e.g. drug-drug, duplicate therapy, etc.."""
    _attribute_docstrings['severity'] = """Indicates the degree of importance associated with the identified issue based on the potential impact on the patient."""
    _attribute_docstrings['patient'] = """Associated patient."""
    _attribute_docstrings['identifiedDateTime'] = """When identified."""
    _attribute_docstrings['identifiedPeriod'] = """When identified."""
    _attribute_docstrings['author'] = """The provider or device that identified the issue."""
    _attribute_docstrings['implicated'] = """Problem resource."""
    _attribute_docstrings['evidence'] = """Supporting evidence."""
    _attribute_docstrings['detail'] = """Description and context."""
    _attribute_docstrings['reference'] = """Authority for issue."""
    _attribute_docstrings['mitigation'] = """Step taken to address."""

    @classmethod
    def attribute_docstrings(cls):
        """Get dict of attributes docstrings."""
        return cls._attribute_docstrings

    _attribute_enums = {}
    """ Dictionary of enum configuration."""
    _attribute_enums['status'] = {
        'url': 'http://hl7.org/fhir/observation-status',
        'restricted_to': ['registered', 'preliminary', 'final', 'amended', 'corrected', 'cancelled', 'entered-in-error', 'unknown'],
        'binding_strength': 'required',
        'class_name': 'str'
    }
    _attribute_enums['severity'] = {
        'url': 'http://hl7.org/fhir/detectedissue-severity',
        'restricted_to': ['high', 'moderate', 'low'],
        'binding_strength': 'required',
        'class_name': 'str'
    }

    @classmethod
    def attribute_enums(cls):
        """Get dict of attributes with enums, Code or CodeableConcept."""
        return cls._attribute_enums

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.identifier = None
        """ Unique id for the detected issue.
        List of `Identifier` items (represented as `dict` in JSON). """
        
        self.status = None
        """ Indicates the status of the detected issue.
        Type `str`. """
        
        self.code = None
        """ Issue Category, e.g. drug-drug, duplicate therapy, etc.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.severity = None
        """ Indicates the degree of importance associated with the identified
        issue based on the potential impact on the patient.
        Type `str`. """
        
        self.patient = None
        """ Associated patient.
        Type `FHIRReference` (represented as `dict` in JSON). """
        
        self.identifiedDateTime = None
        """ When identified.
        Type `FHIRDate` (represented as `str` in JSON). """
        
        self.identifiedPeriod = None
        """ When identified.
        Type `Period` (represented as `dict` in JSON). """
        
        self.author = None
        """ The provider or device that identified the issue.
        Type `FHIRReference` (represented as `dict` in JSON). """
        
        self.implicated = None
        """ Problem resource.
        List of `FHIRReference` items (represented as `dict` in JSON). """
        
        self.evidence = None
        """ Supporting evidence.
        List of `DetectedIssueEvidence` items (represented as `dict` in JSON). """
        
        self.detail = None
        """ Description and context.
        Type `str`. """
        
        self.reference = None
        """ Authority for issue.
        Type `str`. """
        
        self.mitigation = None
        """ Step taken to address.
        List of `DetectedIssueMitigation` items (represented as `dict` in JSON). """
        
        super(DetectedIssue, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(DetectedIssue, self).elementProperties()
        js.extend([
            ("identifier", "identifier", identifier.Identifier, True, None, False),
            ("status", "status", str, False, None, True),
            ("code", "code", codeableconcept.CodeableConcept, False, None, False),
            ("severity", "severity", str, False, None, False),
            ("patient", "patient", fhirreference.FHIRReference, False, None, False),
            ("identifiedDateTime", "identifiedDateTime", fhirdate.FHIRDate, False, "identified", False),
            ("identifiedPeriod", "identifiedPeriod", period.Period, False, "identified", False),
            ("author", "author", fhirreference.FHIRReference, False, None, False),
            ("implicated", "implicated", fhirreference.FHIRReference, True, None, False),
            ("evidence", "evidence", DetectedIssueEvidence, True, None, False),
            ("detail", "detail", str, False, None, False),
            ("reference", "reference", str, False, None, False),
            ("mitigation", "mitigation", DetectedIssueMitigation, True, None, False),
        ])
        return js


from . import backboneelement

class DetectedIssueEvidence(backboneelement.BackboneElement):
    """ Supporting evidence.
    
    Supporting evidence or manifestations that provide the basis for
    identifying the detected issue such as a GuidanceResponse or MeasureReport.
    """

    _attribute_docstrings = {}
    """ Dictionary of attribute documentation."""
    _attribute_docstrings['code'] = """Manifestation."""
    _attribute_docstrings['detail'] = """Supporting information."""

    @classmethod
    def attribute_docstrings(cls):
        """Get dict of attributes docstrings."""
        return cls._attribute_docstrings

    _attribute_enums = {}
    """ Dictionary of enum configuration."""

    @classmethod
    def attribute_enums(cls):
        """Get dict of attributes with enums, Code or CodeableConcept."""
        return cls._attribute_enums

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.code = None
        """ Manifestation.
        List of `CodeableConcept` items (represented as `dict` in JSON). """
        
        self.detail = None
        """ Supporting information.
        List of `FHIRReference` items (represented as `dict` in JSON). """
        
        super(DetectedIssueEvidence, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(DetectedIssueEvidence, self).elementProperties()
        js.extend([
            ("code", "code", codeableconcept.CodeableConcept, True, None, False),
            ("detail", "detail", fhirreference.FHIRReference, True, None, False),
        ])
        return js


class DetectedIssueMitigation(backboneelement.BackboneElement):
    """ Step taken to address.
    
    Indicates an action that has been taken or is committed to reduce or
    eliminate the likelihood of the risk identified by the detected issue from
    manifesting.  Can also reflect an observation of known mitigating factors
    that may reduce/eliminate the need for any action.
    """

    _attribute_docstrings = {}
    """ Dictionary of attribute documentation."""
    _attribute_docstrings['action'] = """What mitigation?."""
    _attribute_docstrings['date'] = """Date committed."""
    _attribute_docstrings['author'] = """Who is committing?."""

    @classmethod
    def attribute_docstrings(cls):
        """Get dict of attributes docstrings."""
        return cls._attribute_docstrings

    _attribute_enums = {}
    """ Dictionary of enum configuration."""

    @classmethod
    def attribute_enums(cls):
        """Get dict of attributes with enums, Code or CodeableConcept."""
        return cls._attribute_enums

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.action = None
        """ What mitigation?.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.date = None
        """ Date committed.
        Type `FHIRDate` (represented as `str` in JSON). """
        
        self.author = None
        """ Who is committing?.
        Type `FHIRReference` (represented as `dict` in JSON). """
        
        super(DetectedIssueMitigation, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(DetectedIssueMitigation, self).elementProperties()
        js.extend([
            ("action", "action", codeableconcept.CodeableConcept, False, None, True),
            ("date", "date", fhirdate.FHIRDate, False, None, False),
            ("author", "author", fhirreference.FHIRReference, False, None, False),
        ])
        return js


import sys
try:
    from . import codeableconcept
except ImportError:
    codeableconcept = sys.modules[__package__ + '.codeableconcept']
try:
    from . import fhirdate
except ImportError:
    fhirdate = sys.modules[__package__ + '.fhirdate']
try:
    from . import fhirreference
except ImportError:
    fhirreference = sys.modules[__package__ + '.fhirreference']
try:
    from . import identifier
except ImportError:
    identifier = sys.modules[__package__ + '.identifier']
try:
    from . import period
except ImportError:
    period = sys.modules[__package__ + '.period']



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 4.0.1-9346c8cc45 (http://hl7.org/fhir/StructureDefinition/Annotation) on 2022-06-22.
#  2022, SMART Health IT.


from . import element

class Annotation(element.Element):
    """ Text node with attribution.
    
    A  text note which also  contains information about who made the statement
    and when.
    """

    _attribute_docstrings = {}
    """ Dictionary of attribute documentation."""
    _attribute_docstrings['authorReference'] = """Individual responsible for the annotation."""
    _attribute_docstrings['authorString'] = """Individual responsible for the annotation."""
    _attribute_docstrings['time'] = """When the annotation was made."""
    _attribute_docstrings['text'] = """The annotation  - text content (as markdown)."""

    @classmethod
    def attribute_docstrings(cls):
        """Get dict of attributes docstrings."""
        return cls._attribute_docstrings

    _attribute_enums = {}
    """ Dictionary of enum configuration."""

    @classmethod
    def attribute_enums(cls):
        """Get dict of attributes with enums, Code or CodeableConcept."""
        return cls._attribute_enums

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.authorReference = None
        """ Individual responsible for the annotation.
        Type `FHIRReference` (represented as `dict` in JSON). """
        
        self.authorString = None
        """ Individual responsible for the annotation.
        Type `str`. """
        
        self.time = None
        """ When the annotation was made.
        Type `FHIRDate` (represented as `str` in JSON). """
        
        self.text = None
        """ The annotation  - text content (as markdown).
        Type `str`. """
        
        super(Annotation, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(Annotation, self).elementProperties()
        js.extend([
            ("authorReference", "authorReference", fhirreference.FHIRReference, False, "author", False),
            ("authorString", "authorString", str, False, "author", False),
            ("time", "time", fhirdate.FHIRDate, False, None, False),
            ("text", "text", str, False, None, True),
        ])
        return js


import sys
try:
    from . import fhirdate
except ImportError:
    fhirdate = sys.modules[__package__ + '.fhirdate']
try:
    from . import fhirreference
except ImportError:
    fhirreference = sys.modules[__package__ + '.fhirreference']



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 4.0.1-9346c8cc45 (http://hl7.org/fhir/StructureDefinition/Practitioner) on 2022-06-22.
#  2022, SMART Health IT.


from . import domainresource

class Practitioner(domainresource.DomainResource):
    """ A person with a  formal responsibility in the provisioning of healthcare or
    related services.
    
    A person who is directly or indirectly involved in the provisioning of
    healthcare.
    """
    
    resource_type = "Practitioner"

    _attribute_docstrings = {}
    """ Dictionary of attribute documentation."""
    _attribute_docstrings['identifier'] = """An identifier for the person as this agent."""
    _attribute_docstrings['active'] = """Whether this practitioner's record is in active use."""
    _attribute_docstrings['name'] = """The name(s) associated with the practitioner."""
    _attribute_docstrings['telecom'] = """A contact detail for the practitioner (that apply to all roles)."""
    _attribute_docstrings['address'] = """Address(es) of the practitioner that are not role specific (typically home address)."""
    _attribute_docstrings['gender'] = """Administrative Gender - the gender that the person is considered to have for administration and record keeping purposes."""
    _attribute_docstrings['birthDate'] = """The date  on which the practitioner was born."""
    _attribute_docstrings['photo'] = """Image of the person."""
    _attribute_docstrings['qualification'] = """Certification, licenses, or training pertaining to the provision of care."""
    _attribute_docstrings['communication'] = """A language the practitioner can use in patient communication."""

    @classmethod
    def attribute_docstrings(cls):
        """Get dict of attributes docstrings."""
        return cls._attribute_docstrings

    _attribute_enums = {}
    """ Dictionary of enum configuration."""
    _attribute_enums['gender'] = {
        'url': 'http://hl7.org/fhir/administrative-gender',
        'restricted_to': ['male', 'female', 'other', 'unknown'],
        'binding_strength': 'required',
        'class_name': 'str'
    }

    @classmethod
    def attribute_enums(cls):
        """Get dict of attributes with enums, Code or CodeableConcept."""
        return cls._attribute_enums

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.identifier = None
        """ An identifier for the person as this agent.
        List of `Identifier` items (represented as `dict` in JSON). """
        
        self.active = None
        """ Whether this practitioner's record is in active use.
        Type `bool`. """
        
        self.name = None
        """ The name(s) associated with the practitioner.
        List of `HumanName` items (represented as `dict` in JSON). """
        
        self.telecom = None
        """ A contact detail for the practitioner (that apply to all roles).
        List of `ContactPoint` items (represented as `dict` in JSON). """
        
        self.address = None
        """ Address(es) of the practitioner that are not role specific
        (typically home address).
        List of `Address` items (represented as `dict` in JSON). """
        
        self.gender = None
        """ Administrative Gender - the gender that the person is considered to
        have for administration and record keeping purposes.
        Type `str`. """
        
        self.birthDate = None
        """ The date  on which the practitioner was born.
        Type `FHIRDate` (represented as `str` in JSON). """
        
        self.photo = None
        """ Image of the person.
        List of `Attachment` items (represented as `dict` in JSON). """
        
        self.qualification = None
        """ Certification, licenses, or training pertaining to the provision of
        care.
        List of `PractitionerQualification` items (represented as `dict` in JSON). """
        
        self.communication = None
        """ A language the practitioner can use in patient communication.
        List of `CodeableConcept` items (represented as `dict` in JSON). """
        
        super(Practitioner, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(Practitioner, self).elementProperties()
        js.extend([
            ("identifier", "identifier", identifier.Identifier, True, None, False),
            ("active", "active", bool, False, None, False),
            ("name", "name", humanname.HumanName, True, None, False),
            ("telecom", "telecom", contactpoint.ContactPoint, True, None, False),
            ("address", "address", address.Address, True, None, False),
            ("gender", "gender", str, False, None, False),
            ("birthDate", "birthDate", fhirdate.FHIRDate, False, None, False),
            ("photo", "photo", attachment.Attachment, True, None, False),
            ("qualification", "qualification", PractitionerQualification, True, None, False),
            ("communication", "communication", codeableconcept.CodeableConcept, True, None, False),
        ])
        return js


from . import backboneelement

class PractitionerQualification(backboneelement.BackboneElement):
    """ Certification, licenses, or training pertaining to the provision of care.
    
    The official certifications, training, and licenses that authorize or
    otherwise pertain to the provision of care by the practitioner.  For
    example, a medical license issued by a medical board authorizing the
    practitioner to practice medicine within a certian locality.
    """

    _attribute_docstrings = {}
    """ Dictionary of attribute documentation."""
    _attribute_docstrings['identifier'] = """An identifier for this qualification for the practitioner."""
    _attribute_docstrings['code'] = """Coded representation of the qualification."""
    _attribute_docstrings['period'] = """Period during which the qualification is valid."""
    _attribute_docstrings['issuer'] = """Organization that regulates and issues the qualification."""

    @classmethod
    def attribute_docstrings(cls):
        """Get dict of attributes docstrings."""
        return cls._attribute_docstrings

    _attribute_enums = {}
    """ Dictionary of enum configuration."""

    @classmethod
    def attribute_enums(cls):
        """Get dict of attributes with enums, Code or CodeableConcept."""
        return cls._attribute_enums

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.
        
        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """
        
        self.identifier = None
        """ An identifier for this qualification for the practitioner.
        List of `Identifier` items (represented as `dict` in JSON). """
        
        self.code = None
        """ Coded representation of the qualification.
        Type `CodeableConcept` (represented as `dict` in JSON). """
        
        self.period = None
        """ Period during which the qualification is valid.
        Type `Period` (represented as `dict` in JSON). """
        
        self.issuer = None
        """ Organization that regulates and issues the qualification.
        Type `FHIRReference` (represented as `dict` in JSON). """
        
        super(PractitionerQualification, self).__init__(jsondict=jsondict, strict=strict)
    
    def elementProperties(self):
        js = super(PractitionerQualification, self).elementProperties()
        js.extend([
            ("identifier", "identifier", identifier.Identifier, True, None, False),
            ("code", "code", codeableconcept.CodeableConcept, False, None, True),
            ("period", "period", period.Period, False, None, False),
            ("issuer", "issuer", fhirreference.FHIRReference, False, None, False),
        ])
        return js


import sys
try:
    from . import address
except ImportError:
    address = sys.modules[__package__ + '.address']
try:
    from . import attachment
except ImportError:
    attachment = sys.modules[__package__ + '.attachment']
try:
    from . import codeableconcept
except ImportError:
    codeableconcept = sys.modules[__package__ + '.codeableconcept']
try:
    from . import contactpoint
except ImportError:
    contactpoint = sys.modules[__package__ + '.contactpoint']
try:
    from . import fhirdate
except ImportError:
    fhirdate = sys.modules[__package__ + '.fhirdate']
try:
    from . import fhirreference
except ImportError:
    fhirreference = sys.modules[__package__ + '.fhirreference']
try:
    from . import humanname
except ImportError:
    humanname = sys.modules[__package__ + '.humanname']
try:
    from . import identifier
except ImportError:
    identifier = sys.modules[__package__ + '.identifier']
try:
    from . import period
except ImportError:
    period = sys.modules[__package__ + '.period']
